#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from tacker.vnfm.monitor_drivers.zabbix import zabbix
import testtools
from unittest import mock


class TestVNFMonitorZabbix(testtools.TestCase):

    def setUp(self):
        super(TestVNFMonitorZabbix, self).setUp()
        zabbix.VNFMonitorZabbix.tacker_token = 'a1b2c3d4e5'
        self.monitor_zabbix = zabbix.VNFMonitorZabbix()

    @mock.patch('tacker.vnfm.monitor_drivers.zabbix.zabbix.'
                'VNFMonitorZabbix.add_to_appmonitor')
    def test_add_to_appmonitor(self, mock_ac):
        mock_ac.return_value = None

        test_vnf = {'vnfd': {'tenant_id': 'd1e6919c73074d18ab6cd49a02e08391'},
                    'id': 'b9af3cb5-6e43-4b2c-a056-67bda3f71e1a'}
        test_kwargs = {'vdus': {'VDU1':
                                {'parameters':
                                 {'application':
                                  {'app_name': 'apache2',
                                   'app_status': {'actionname': 'cmd',
                                                  'cmd-action': 'sudo service \
                                                  apache2 restart',
                                                  'condition': ['down']},
                                   'ssh_username': 'ubuntu',
                                   'app_port': 80,
                                   'ssh_password': 'ubuntu'}},
                                 'name': 'zabbix',
                                 'zabbix_username': 'Admin',
                                 'zabbix_password': 'zabbix',
                                 'zabbix_server_ip': '192.168.11.53',
                                 'zabbix_server_port': 80,
                                 'mgmt_ip': '192.168.11.206'}}}

        monitor_return = self.monitor_zabbix.\
            add_to_appmonitor(test_kwargs, test_vnf)
        self.assertEqual(None, monitor_return)



# Copyright 2016 - Nokia
# Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import codecs
import os

import testtools
import yaml

from tacker.extensions import vnfm
from tacker.tosca import utils as toscautils
from toscaparser import tosca_template
from translator.hot import tosca_translator


def _get_template(name):
    filename = os.path.join(
        os.path.dirname(os.path.abspath(__file__)),
        "../infra_drivers/openstack/data/", name)
    with codecs.open(filename, encoding='utf-8', errors='strict') as f:
        return f.read()


class TestToscaUtils(testtools.TestCase):
    tosca_openwrt = _get_template('test_tosca_openwrt.yaml')
    vnfd_dict = yaml.safe_load(tosca_openwrt)
    toscautils.updateimports(vnfd_dict)

    def setUp(self):
        super(TestToscaUtils, self).setUp()
        self.tosca = tosca_template.ToscaTemplate(
            parsed_params={}, a_file=False, yaml_dict_tpl=self.vnfd_dict)
        self.tosca_flavor = _get_template('test_tosca_flavor.yaml')

    def test_updateimport(self):
        importspath = os.path.abspath('./tacker/tosca/lib/')
        file1 = importspath + '/tacker_defs.yaml'
        file2 = importspath + '/tacker_nfv_defs.yaml'
        expected_imports = [file1, file2]
        self.assertEqual(expected_imports, self.vnfd_dict['imports'])

    def test_get_mgmt_driver(self):
        expected_mgmt_driver = 'openwrt'
        mgmt_driver = toscautils.get_mgmt_driver(self.tosca)
        self.assertEqual(expected_mgmt_driver, mgmt_driver)

    def test_get_vdu_monitoring(self):
        expected_monitoring = {'vdus': {'VDU1': {'ping': {
                               'actions':
                               {'failure': 'respawn'},
                               'name': 'ping',
                               'parameters': {'count': 3,
                                              'interval': 10},
                               'monitoring_params': {'count': 3,
                                                  'interval': 10}}}}}
        monitoring = toscautils.get_vdu_monitoring(self.tosca)
        self.assertEqual(expected_monitoring, monitoring)

    def test_get_mgmt_ports(self):
        expected_mgmt_ports = {'mgmt_ip-VDU1': 'CP1'}
        mgmt_ports = toscautils.get_mgmt_ports(self.tosca)
        self.assertEqual(expected_mgmt_ports, mgmt_ports)

    def test_post_process_template(self):
        tosca_post_process_tpl = _get_template(
            'test_tosca_post_process_template.yaml')
        vnfd_dict = yaml.safe_load(tosca_post_process_tpl)
        toscautils.updateimports(vnfd_dict)
        tosca = tosca_template.ToscaTemplate(parsed_params={}, a_file=False,
                                             yaml_dict_tpl=vnfd_dict)
        toscautils.post_process_template(tosca)

        invalidNodes = 0
        deletedProperties = 0
        convertedValues = 0
        convertedProperties = 0

        for nt in tosca.nodetemplates:
            if (nt.type_definition.is_derived_from(toscautils.MONITORING) or
                    nt.type_definition.is_derived_from(toscautils.FAILURE) or
                    nt.type_definition.is_derived_from(toscautils.PLACEMENT)):
                invalidNodes += 1

            if nt.type in toscautils.delpropmap:
                for prop in toscautils.delpropmap[nt.type]:
                    for p in nt.get_properties_objects():
                        if prop == p.name:
                            deletedProperties += 1

            if nt.type in toscautils.convert_prop_values:
                for prop in toscautils.convert_prop_values[nt.type]:
                    convertmap = toscautils.convert_prop_values[nt.type][prop]
                    for p in nt.get_properties_objects():
                        if (prop == p.name and
                                p.value in convertmap):
                            convertedValues += 1

            if nt.type in toscautils.convert_prop:
                for prop in toscautils.convert_prop[nt.type]:
                    for p in nt.get_properties_objects():
                        if prop == p.name:
                            convertedProperties += 1

            if nt.name == 'VDU1':
                vdu1_hints = nt.get_properties().get('scheduler_hints')
                vdu1_rsv = vdu1_hints.value.get('reservation')

        self.assertEqual(0, invalidNodes)
        self.assertEqual(0, deletedProperties)
        self.assertEqual(0, convertedValues)
        self.assertEqual(0, convertedProperties)
        self.assertEqual(vdu1_rsv, '459e94c9-efcd-4320-abf5-8c18cd82c331')

    def test_post_process_heat_template(self):
        tosca1 = tosca_template.ToscaTemplate(parsed_params={}, a_file=False,
                          yaml_dict_tpl=self.vnfd_dict)
        toscautils.post_process_template(tosca1)
        translator = tosca_translator.TOSCATranslator(tosca1, {})
        heat_template_yaml = translator.translate()
        expected_heat_tpl = _get_template('hot_tosca_openwrt.yaml')
        mgmt_ports = toscautils.get_mgmt_ports(self.tosca)
        heat_tpl = toscautils.post_process_heat_template(
            heat_template_yaml, mgmt_ports, {}, {}, {})

        heatdict = yaml.safe_load(heat_tpl)
        expecteddict = yaml.safe_load(expected_heat_tpl)
        self.assertEqual(expecteddict, heatdict)

    def test_findvdus(self):
        vdus = toscautils.findvdus(self.tosca)

        self.assertEqual(1, len(vdus))

        for vdu in vdus:
            self.assertEqual(True, vdu.type_definition.is_derived_from(
                toscautils.TACKERVDU))

    def test_get_flavor_dict(self):
        vnfd_dict = yaml.safe_load(self.tosca_flavor)
        toscautils.updateimports(vnfd_dict)
        tosca = tosca_template.ToscaTemplate(a_file=False,
                                             yaml_dict_tpl=vnfd_dict)
        expected_flavor_dict = {
            "VDU1": {
                "vcpus": 2,
                "disk": 10,
                "ram": 512
            }
        }
        actual_flavor_dict = toscautils.get_flavor_dict(tosca)
        self.assertEqual(expected_flavor_dict, actual_flavor_dict)

    def test_add_resources_tpl_for_flavor(self):
        dummy_heat_dict = yaml.safe_load(_get_template(
            'hot_flavor_and_capabilities.yaml'))
        expected_dict = yaml.safe_load(_get_template('hot_flavor.yaml'))
        dummy_heat_res = {
            "flavor": {
                "VDU1": {
                    "vcpus": 2,
                    "ram": 512,
                    "disk": 10
                }
            }
        }
        toscautils.add_resources_tpl(dummy_heat_dict, dummy_heat_res)
        self.assertEqual(expected_dict, dummy_heat_dict)

    def test_get_flavor_dict_extra_specs_all_numa_count(self):
        tosca_fes_all_numa_count = _get_template(
            'tosca_flavor_all_numa_count.yaml')
        vnfd_dict = yaml.safe_load(tosca_fes_all_numa_count)
        toscautils.updateimports(vnfd_dict)
        tosca = tosca_template.ToscaTemplate(a_file=False,
                                             yaml_dict_tpl=vnfd_dict)
        expected_flavor_dict = {
            "VDU1": {
                "vcpus": 8,
                "disk": 10,
                "ram": 4096,
                "extra_specs": {
                    'hw:cpu_policy': 'dedicated', 'hw:mem_page_size': 'any',
                    'hw:cpu_sockets': 2, 'hw:cpu_threads': 2,
                    'hw:numa_nodes': 2, 'hw:cpu_cores': 2,
                    'hw:cpu_threads_policy': 'avoid'
                }
            }
        }
        actual_flavor_dict = toscautils.get_flavor_dict(tosca)
        self.assertEqual(expected_flavor_dict, actual_flavor_dict)

    def test_get_flavor_dict_with_wrong_cpu(self):
        tosca_fes = _get_template(
            'tosca_flavor_with_wrong_cpu.yaml')
        vnfd_dict = yaml.safe_load(tosca_fes)
        toscautils.updateimports(vnfd_dict)
        tosca = tosca_template.ToscaTemplate(a_file=False,
                                             yaml_dict_tpl=vnfd_dict)

        self.assertRaises(vnfm.CpuAllocationInvalidValues,
                          toscautils.get_flavor_dict,
                          tosca)

    def test_tacker_conf_heat_extra_specs_all_numa_count(self):
        tosca_fes_all_numa_count = _get_template(
            'tosca_flavor_all_numa_count.yaml')
        vnfd_dict = yaml.safe_load(tosca_fes_all_numa_count)
        toscautils.updateimports(vnfd_dict)
        tosca = tosca_template.ToscaTemplate(a_file=False,
                                             yaml_dict_tpl=vnfd_dict)
        expected_flavor_dict = {
            "VDU1": {
                "vcpus": 8,
                "disk": 10,
                "ram": 4096,
                "extra_specs": {
                    'hw:cpu_policy': 'dedicated', 'hw:mem_page_size': 'any',
                    'hw:cpu_sockets': 2, 'hw:cpu_threads': 2,
                    'hw:numa_nodes': 2, 'hw:cpu_cores': 2,
                    'hw:cpu_threads_policy': 'avoid',
                    'aggregate_instance_extra_specs:nfv': 'true'
                }
            }
        }
        actual_flavor_dict = toscautils.get_flavor_dict(
            tosca, {"aggregate_instance_extra_specs:nfv": "true"})
        self.assertEqual(expected_flavor_dict, actual_flavor_dict)

    def test_add_resources_tpl_for_image(self):
        dummy_heat_dict = yaml.safe_load(_get_template(
            'hot_image_before_processed_image.yaml'))
        expected_dict = yaml.safe_load(_get_template(
            'hot_image_after_processed_image.yaml'))
        dummy_heat_res = {
            "image": {
                "VDU1": {
                    "location": "http://URL/v1/openwrt.qcow2",
                    "container_format": "bare",
                    "disk_format": "raw"
                }
            }
        }
        toscautils.add_resources_tpl(dummy_heat_dict, dummy_heat_res)
        self.assertEqual(expected_dict, dummy_heat_dict)

    def test_convert_unsupported_res_prop_kilo_ver(self):
        unsupported_res_prop_dict = {'OS::Neutron::Port': {
            'port_security_enabled': 'value_specs', }, }
        dummy_heat_dict = yaml.safe_load(_get_template(
            'hot_tosca_openwrt.yaml'))
        expected_heat_dict = yaml.safe_load(_get_template(
            'hot_tosca_openwrt_kilo.yaml'))
        toscautils.convert_unsupported_res_prop(dummy_heat_dict,
                                                unsupported_res_prop_dict)
        self.assertEqual(expected_heat_dict, dummy_heat_dict)

    def test_check_for_substitution_mappings(self):
        tosca_sb_map = _get_template('../../../../../etc/samples/test-nsd-'
                                     'vnfd1.yaml')
        param = {'substitution_mappings': {
                 'VL2': {'type': 'tosca.nodes.nfv.VL', 'properties': {
                         'network_name': 'net0', 'vendor': 'tacker'}},
                 'VL1': {'type': 'tosca.nodes.nfv.VL', 'properties': {
                         'network_name': 'net_mgmt', 'vendor': 'tacker'}},
                 'requirements': {'virtualLink2': 'VL2',
                                  'virtualLink1': 'VL1'}}}
        template = yaml.safe_load(tosca_sb_map)
        toscautils.updateimports(template)
        toscautils.check_for_substitution_mappings(template, param)
        self.assertNotIn('substitution_mappings', param)

    def test_get_block_storage_details(self):
        tosca_vol = _get_template('tosca_block_storage.yaml')
        vnfd_dict = yaml.safe_load(tosca_vol)
        expected_dict = {
            'volumes': {
                'VB1': {
                    'image': 'cirros-0.5.2-x86_64-disk',
                    'size': '1'
                }
            },
            'volume_attachments': {
                'CB1': {
                    'instance_uuid': {'get_resource': 'VDU1'},
                    'mountpoint': '/dev/vdb',
                    'volume_id': {'get_resource': 'VB1'}}
            }
        }
        volume_details = toscautils.get_block_storage_details(vnfd_dict)
        self.assertEqual(expected_dict, volume_details)

    def test_get_block_storage_details_volume_id(self):
        tosca_vol = _get_template(
            'test-tosca-vnfd-existing-block-storage.yaml')
        vnfd_dict = yaml.safe_load(tosca_vol)
        expected_dict = {
            'volumes': {
                'VB1': {
                    'volume_id': 'my_vol'
                }
            },
            'volume_attachments': {
                'CB1': {
                    'instance_uuid': {'get_resource': 'VDU1'},
                    'mountpoint': '/dev/vdb',
                    'volume_id': {'get_param': 'my_vol'}}
            }
        }
        volume_details = toscautils.get_block_storage_details(vnfd_dict)
        self.assertEqual(expected_dict, volume_details)



# Copyright 2013 Intel Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""add tables for tacker framework

Revision ID: 1c6b0d82afcd
Revises: 2db5203cb7a9
Create Date: 2013-11-25 18:06:13.980301

"""

# flake8: noqa: E402

# revision identifiers, used by Alembic.
revision = '1c6b0d82afcd'
down_revision = None

from alembic import op
import sqlalchemy as sa


def upgrade(active_plugins=None, options=None):
    op.create_table(
        'devicetemplates',
        sa.Column('id', sa.String(length=36), nullable=False),
        sa.Column('tenant_id', sa.String(length=255), nullable=True),
        sa.Column('name', sa.String(length=255), nullable=True),
        sa.Column('description', sa.String(length=255), nullable=True),
        sa.Column('infra_driver', sa.String(length=255), nullable=True),
        sa.Column('mgmt_driver', sa.String(length=255), nullable=True),
        sa.PrimaryKeyConstraint('id'),
    )
    op.create_table(
        'devicetemplateattributes',
        sa.Column('id', sa.String(length=36), nullable=False),
        sa.Column('template_id', sa.String(length=36), nullable=False),
        sa.Column('key', sa.String(length=255), nullable=False),
        sa.Column('value', sa.String(length=4096), nullable=True),
        sa.ForeignKeyConstraint(['template_id'], ['devicetemplates.id'], ),
        sa.PrimaryKeyConstraint('id'),
    )
    op.create_table(
        'devices',
        sa.Column('id', sa.String(length=255), nullable=False),
        sa.Column('tenant_id', sa.String(length=255), nullable=True),
        sa.Column('name', sa.String(length=255), nullable=True),
        sa.Column('template_id', sa.String(length=36), nullable=True),
        sa.Column('instance_id', sa.String(length=255), nullable=True),
        sa.Column('mgmt_url', sa.String(length=255), nullable=True),
        sa.Column('status', sa.String(length=255), nullable=True),
        sa.ForeignKeyConstraint(['template_id'], ['devicetemplates.id'], ),
        sa.PrimaryKeyConstraint('id'),
    )
    op.create_table(
        'deviceattributes',
        sa.Column('id', sa.String(length=36), nullable=False),
        sa.Column('device_id', sa.String(length=255)),
        sa.Column('key', sa.String(length=255), nullable=False),
        sa.Column('value', sa.String(length=4096), nullable=True),
        sa.ForeignKeyConstraint(['device_id'], ['devices.id'], ),
        sa.PrimaryKeyConstraint('id'),
    )



# Copyright (C) 2019 NTT DATA
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import ddt
from http import client as http_client
import json
import os
from unittest import mock
import urllib
from webob import exc

from oslo_serialization import jsonutils

from tacker.api.vnfpkgm.v1 import controller
from tacker.common import exceptions as tacker_exc
from tacker.conductor.conductorrpc.vnf_pkgm_rpc import VNFPackageRPCAPI
from tacker.glance_store import store as glance_store
from tacker import objects
from tacker.objects import fields
from tacker.objects import vnf_package
from tacker.objects.vnf_package import VnfPackagesList
from tacker.objects.vnf_software_image import VnfSoftwareImage
from tacker.tests import constants
from tacker.tests.unit import base
from tacker.tests.unit import fake_request
from tacker.tests.unit.vnfpkgm import fakes


@ddt.ddt
class TestController(base.TestCase):

    def setUp(self):
        super(TestController, self).setUp()
        self.controller = controller.VnfPkgmController()

    @property
    def app(self):
        return fakes.wsgi_app_v1()

    def _make_problem_detail(self, title, detail, status):
        res = exc.Response(content_type='application/problem+json')
        problemDetails = {}
        problemDetails['title'] = title
        problemDetails['detail'] = detail
        problemDetails['status'] = status
        res.text = json.dumps(problemDetails)
        res.status_int = status
        return res

    @mock.patch.object(vnf_package, '_vnf_package_create')
    @mock.patch.object(vnf_package.VnfPackage, '_from_db_object')
    def test_create_with_status_201(self, mock_from_db, mock_vnf_pack):
        body = {'userDefinedData': {'abc': 'xyz'}}
        req = fake_request.HTTPRequest.blank('/vnf_packages')
        req.body = jsonutils.dump_as_bytes(body)
        req.headers['Content-Type'] = 'application/json'
        req.method = 'POST'
        resp = req.get_response(self.app)
        location_pattern = (r'http://localhost/vnfpkgm/v1/vnf_packages/'
                            r'([a-f]|\d|-){36}')
        self.assertRegex(resp.headers['location'], location_pattern)
        self.assertEqual(http_client.CREATED, resp.status_code)

    @mock.patch.object(vnf_package, '_vnf_package_create')
    @mock.patch.object(vnf_package.VnfPackage, '_from_db_object')
    def test_create_without_userdefine_data(self, mock_from_db,
                                            mock_vnf_pack):
        body = {'userDefinedData': {}}
        req = fake_request.HTTPRequest.blank('/vnf_packages')
        req.body = jsonutils.dump_as_bytes(body)
        req.headers['Content-Type'] = 'application/json'
        req.method = 'POST'
        resp = req.get_response(self.app)
        location_pattern = (r'http://localhost/vnfpkgm/v1/vnf_packages/'
                            r'([a-f]|\d|-){36}')
        self.assertRegex(resp.headers['location'], location_pattern)
        self.assertEqual(http_client.CREATED, resp.status_code)

    @mock.patch.object(VnfSoftwareImage, 'get_by_id')
    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_show(self, mock_vnf_by_id, mock_sw_image_by_id):
        req = fake_request.HTTPRequest.blank(
            '/vnfpkgm/v1/vnf_packages/%s' % constants.UUID)
        mock_vnf_by_id.return_value = fakes.return_vnfpkg_obj()
        mock_sw_image_by_id.return_value = fakes.return_software_image()
        expected_result = fakes.VNFPACKAGE_RESPONSE
        res_dict = self.controller.show(req, constants.UUID)
        self.assertEqual(expected_result, res_dict)

    def test_show_with_invalid_uuid(self):
        req = fake_request.HTTPRequest.blank(
            '/vnfpkgm/v1/vnf_packages/%s' % constants.INVALID_UUID)
        self.assertRaises(exc.HTTPNotFound, self.controller.show,
                          req, constants.INVALID_UUID)

    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_show_no_vnf_package(self, mock_vnf_by_id):
        req = fake_request.HTTPRequest.blank(
            '/vnfpkgm/v1/vnf_packages/%s' % constants.UUID)
        msg = _("Can not find requested vnf package: %s") % constants.UUID
        mock_vnf_by_id.side_effect = exc.HTTPNotFound(explanation=msg)
        self.assertRaises(exc.HTTPNotFound, self.controller.show,
                          req, constants.UUID)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    @ddt.data('/vnfpkgm/v1/vnf_packages')
    def test_index(self, path, mock_vnf_list):
        req = fake_request.HTTPRequest.blank(path)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        res_dict = self.controller.index(req)
        expected_result = fakes.index_response(
            remove_attrs=[
                'softwareImages',
                'checksum',
                'userDefinedData',
                'additionalArtifacts'])
        self.assertEqual(expected_result, res_dict)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    def test_index_attribute_selector_all_fields(self, mock_vnf_list):
        params = {'all_fields': ''}
        query = urllib.parse.urlencode(params)
        req = fake_request.HTTPRequest.blank('/vnfpkgm/v1/vnf_packages?' +
            query)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        res_dict = self.controller.index(req)
        expected_result = fakes.index_response()
        self.assertEqual(expected_result, res_dict)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    def test_index_attribute_selector_exclude_default(self, mock_vnf_list):
        params = {'exclude_default': ''}
        query = urllib.parse.urlencode(params)
        req = fake_request.HTTPRequest.blank('/vnfpkgm/v1/vnf_packages?' +
            query)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        res_dict = self.controller.index(req)
        expected_result = fakes.index_response(
            remove_attrs=[
                'softwareImages',
                'checksum',
                'userDefinedData',
                'additionalArtifacts'])
        self.assertEqual(expected_result, res_dict)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    @ddt.data(
        {'exclude_fields': 'softwareImages'},
        {'exclude_fields': 'checksum'},
        {'exclude_fields': 'userDefinedData'},
        {'exclude_fields': 'additionalArtifacts'}
    )
    def test_index_attribute_selector_exclude_fields(self, params,
            mock_vnf_list):
        query = urllib.parse.urlencode(params)
        req = fake_request.HTTPRequest.blank('/vnfpkgm/v1/vnf_packages?' +
            query)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        res_dict = self.controller.index(req)
        remove_attrs = [params['exclude_fields']]
        expected_result = fakes.index_response(remove_attrs=remove_attrs)
        self.assertEqual(expected_result, res_dict)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    @ddt.data(
        {'fields': 'softwareImages'},
        {'fields': 'checksum'},
        {'fields': 'userDefinedData'},
        {'fields': 'additionalArtifacts'}
    )
    def test_index_attribute_selector_fields(self, params, mock_vnf_list):
        """Test valid attribute names with fields parameter

        We can specify complex attributes in fields. Hence the data only
        contains such attributes.
        """
        complex_attrs = [
            'softwareImages',
            'checksum',
            'userDefinedData',
            'additionalArtifacts']
        query = urllib.parse.urlencode(params)
        req = fake_request.HTTPRequest.blank('/vnfpkgm/v1/vnf_packages?' +
                query)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        res_dict = self.controller.index(req)
        remove_attrs = [x for x in complex_attrs if x != params['fields']]
        expected_result = fakes.index_response(remove_attrs=remove_attrs)
        self.assertEqual(expected_result, res_dict)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    def test_index_attribute_selector_user_defined_data_combination(self,
            mock_vnf_list):
        """Query user defined data with fields parameter

        This test queries combination of user defined data. i.e. fields of
        different complex attributes.
        """
        params = {
            'fields': 'userDefinedData/key1,softwareImages/userMetadata/key3',
        }
        query = urllib.parse.urlencode(params)
        req = fake_request.HTTPRequest.blank('/vnfpkgm/v1/vnf_packages?' +
            query)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        res_dict = self.controller.index(req)
        vnf_package_updates = {
            'userDefinedData': {'key1': 'value1'},
            'softwareImages': [{'userMetadata': {'key3': 'value3'}}]
        }
        expected_result = fakes.index_response(
            remove_attrs=[
                'checksum',
                'additionalArtifacts'],
            vnf_package_updates=vnf_package_updates)
        self.assertEqual(expected_result, res_dict)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    def test_index_attribute_selector_user_defined_data(self, mock_vnf_list):
        params = {'fields': 'userDefinedData/key1,userDefinedData/key2'}
        query = urllib.parse.urlencode(params)
        req = fake_request.HTTPRequest.blank('/vnfpkgm/v1/vnf_packages?' +
            query)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        res_dict = self.controller.index(req)
        expected_result = fakes.index_response(remove_attrs=[
            'checksum', 'softwareImages', 'additionalArtifacts'])
        self.assertEqual(expected_result, res_dict)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    def test_index_attribute_selector_nested_complex_attribute(self,
            mock_vnf_list):
        params = {'fields': 'softwareImages/checksum/algorithm,'
            'softwareImages/minRam,additionalArtifacts/metadata,'
            'additionalArtifacts/checksum/algorithm'}
        query = urllib.parse.urlencode(params)
        req = fake_request.HTTPRequest.blank('/vnfpkgm/v1/vnf_packages?' +
            query)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        res_dict = self.controller.index(req)
        vnf_package_updates = {
            'softwareImages': [{
                'minRam': 0,
                'checksum': {'algorithm': 'fake-algorithm'}
            }],
            'additionalArtifacts': [{
                'metadata': {},
                'checksum': {'algorithm': 'SHA-256'}
            }]
        }
        expected_result = fakes.index_response(remove_attrs=[
            'checksum', 'userDefinedData'],
            vnf_package_updates=vnf_package_updates)
        self.assertEqual(expected_result, res_dict)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    @ddt.data(
        {'filter': '(eq,vnfdId,%s)' % constants.UUID},
        {'filter': '(in,vnfdId,%s)' % constants.UUID},
        {'filter': '(cont,vnfdId,%s)' % constants.UUID},
        {'filter': '(neq,vnfdId,%s)' % constants.UUID},
        {'filter': '(nin,vnfdId,%s)' % constants.UUID},
        {'filter': '(ncont,vnfdId,%s)' % constants.UUID},
        {'filter': '(gt,softwareImages/createdAt,2020-03-11 04:10:15+00:00)'},
        {'filter': '(gte,softwareImages/createdAt,2020-03-14 04:10:15+00:00)'},
        {'filter': '(lt,softwareImages/createdAt,2020-03-20 04:10:15+00:00)'},
        {'filter': '(lte,softwareImages/createdAt,2020-03-11 04:10:15+00:00)'},
        {'filter': '(eq,additionalArtifacts/checksum/algorithm,'
                   'SHA-256)'},
        {'filter': '(neq,additionalArtifacts/checksum/algorithm,'
                   'SHA-256)'},
        {'filter': '(in,additionalArtifacts/checksum/algorithm,'
                   'SHA-256)'},
        {'filter': '(nin,additionalArtifacts/checksum/algorithm,'
                   'SHA-256)'},
        {'filter': '(cont,additionalArtifacts/checksum/algorithm,'
                   'SHA-256)'},
        {'filter': '(ncont,additionalArtifacts/checksum/algorithm,'
                   'SHA-256)'})
    def test_index_filter_operator(self, filter_params, mock_vnf_list):
        """Tests all supported operators in filter expression """
        query = urllib.parse.urlencode(filter_params)
        req = fake_request.HTTPRequest.blank('/vnfpkgm/v1/vnf_packages?' +
            query)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        res_dict = self.controller.index(req)
        expected_result = fakes.index_response(
            remove_attrs=[
                'softwareImages',
                'checksum',
                'userDefinedData',
                'additionalArtifacts'])
        self.assertEqual(expected_result, res_dict)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    def test_index_filter_combination(self, mock_vnf_list):
        """Test multiple filter parameters separated by semicolon """
        params = {'filter': '(eq,vnfdId,%s);(eq,id,%s)' %
                  (constants.UUID, constants.UUID)}
        query = urllib.parse.urlencode(params)
        req = fake_request.HTTPRequest.blank('/vnfpkgm/v1/vnf_packages?' +
            query)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        res_dict = self.controller.index(req)
        expected_result = fakes.index_response(
            remove_attrs=[
                'softwareImages',
                'checksum',
                'userDefinedData',
                'additionalArtifacts'])
        self.assertEqual(expected_result, res_dict)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    @ddt.data(
        {'filter': '(eq,id,%s)' % constants.UUID},
        {'filter': '(eq,vnfdId,%s)' % constants.UUID},
        {'filter': '(eq,onboardingState,ONBOARDED)'},
        {'filter': '(eq,operationalState,ENABLED)'},
        {'filter': '(eq,usageState,NOT_IN_USE)'},
        {'filter': '(eq,vnfProvider,dummy_value)'},
        {'filter': '(eq,vnfProductName,dummy_value)'},
        {'filter': '(eq,vnfSoftwareVersion,dummy_value)'},
        {'filter': '(eq,vnfdVersion,dummy_value)'},
        {'filter': '(eq,userDefinedData/key1,dummy_value)'},
        {'filter': '(eq,checksum/algorithm,dummy_value)'},
        {'filter': '(eq,checksum/hash,dummy_value)'},
        {'filter': '(eq,softwareImages/id,%s)' % constants.UUID},
        {'filter': '(eq,softwareImages/imagePath,dummy_value)'},
        {'filter': '(eq,softwareImages/diskFormat,dummy_value)'},
        {'filter': '(eq,softwareImages/userMetadata/key3,dummy_value)'},
        {'filter': '(eq,softwareImages/size,0)'},
        {'filter': '(gt,softwareImages/createdAt,2020-03-14 04:10:15+00:00)'},
        {'filter': '(eq,softwareImages/name,dummy_value)'},
        {'filter': '(eq,softwareImages/minDisk,0)'},
        {'filter': '(eq,softwareImages/version,dummy_value)'},
        {'filter': '(eq,softwareImages/provider,dummy_value)'},
        {'filter': '(eq,softwareImages/minRam,0)'},
        {'filter': '(eq,softwareImages/containerFormat,dummy_value)'},
        {'filter': '(eq,softwareImages/checksum/hash,dummy_value)'},
        {'filter': '(eq,softwareImages/checksum/algorithm,dummy_value)'},
        {'filter': '(eq,additionalArtifacts/artifactPath,dummy_value)'},
        {'filter': '(eq,additionalArtifacts/checksum/algorithm,dummy_value)'},
        {'filter': '(eq,additionalArtifacts/checksum/hash,dummy_value)'}
    )
    def test_index_filter_attributes(self, filter_params, mock_vnf_list):
        """Test various attributes supported for filter parameter """
        query = urllib.parse.urlencode(filter_params)
        req = fake_request.HTTPRequest.blank('/vnfpkgm/v1/vnf_packages?' +
            query)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        res_dict = self.controller.index(req)
        expected_result = fakes.index_response(
            remove_attrs=[
                'softwareImages',
                'checksum',
                'userDefinedData',
                'additionalArtifacts'])
        self.assertEqual(expected_result, res_dict)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    @ddt.data(
        {'filter': "(eq,vnfProductName,dummy_value)"},
        {'filter': "(eq,vnfProductName,dummy value)"},
        {'filter': "(eq,vnfProductName,'dummy value')"},
        {'filter': "(eq,vnfProductName,'dummy (hi) value')"},
        {'filter': "(eq,vnfProductName,'dummy ''hi'' value')"},
        {'filter': "(eq,vnfProductName,'''dummy ''hi'' value''')"},
    )
    def test_index_filter_valid_string_values(self, filter_params,
            mock_vnf_list):
        """Tests all the supported string values.

        For example:
        - values which are not enclosed in single quotes
        - values which are enclosed in single quotes
        - values having single quotes within them
        - values having round brackets in them
        """
        query = urllib.parse.urlencode(filter_params)
        req = fake_request.HTTPRequest.blank('/vnfpkgm/v1/vnf_packages?' +
            query)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        res_dict = self.controller.index(req)
        expected_result = fakes.index_response(
            remove_attrs=[
                'softwareImages',
                'checksum',
                'userDefinedData',
                'additionalArtifacts'])
        self.assertEqual(expected_result, res_dict)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    @ddt.data(
        {'filter': "(eq,vnfProductName,value"},
        {'filter': "eq,vnfProductName,value)"},
        {'filter': "(eq,vnfProductName,value);"},
        {'filter': "(eq , vnfProductName ,value)"},
    )
    def test_index_filter_invalid_expression(self, filter_params,
            mock_vnf_list):
        """Test invalid filter expression """
        query = urllib.parse.urlencode(filter_params)
        req = fake_request.HTTPRequest.blank('/vnfpkgm/v1/vnf_packages?' +
            query)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        self.assertRaises(tacker_exc.ValidationError, self.controller.index,
                          req)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    @ddt.data(
        {'filter': "(eq,vnfProductName,singl'quote)"},
        {'filter': "(eq,vnfProductName,three''' quotes)"},
        {'filter': "(eq,vnfProductName,round ) bracket)"},
        {'filter': "(eq,vnfProductName,'dummy 'hi' value')"},
        {'filter': "(eq,vnfProductName,'dummy's value')"},
        {'filter': "(eq,vnfProductName,'three ''' quotes')"},
    )
    def test_index_filter_invalid_string_values(self, filter_params,
            mock_vnf_list):
        """Test invalid string values as per ETSI NFV SOL013 5.2.2 """
        query = urllib.parse.urlencode(filter_params)
        req = fake_request.HTTPRequest.blank('/vnfpkgm/v1/vnf_packages?' +
            query)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        self.assertRaises(tacker_exc.ValidationError, self.controller.index,
                          req)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    @ddt.data(
        {'filter': '(eq,vnfdId,value1,value2)'},
        {'filter': '(fake,vnfdId,dummy_vnfd_id)'},
        {'filter': '(,vnfdId,dummy_vnfd_id)'},
    )
    def test_index_filter_invalid_operator(self, params, mock_vnf_list):
        """Test invalid operator in filter expression """
        query = urllib.parse.urlencode(params)
        req = fake_request.HTTPRequest.blank('/vnfpkgm/v1/vnf_packages?' +
            query)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        self.assertRaises(tacker_exc.ValidationError, self.controller.index,
                          req)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    @ddt.data(
        {'filter': '(eq,fakeattr,fakevalue)'},
        {'filter': '(eq,,fakevalue)'},
    )
    def test_index_filter_invalid_attribute(self, params, mock_vnf_list):
        """Test invalid attribute in filter expression """
        query = urllib.parse.urlencode(params)
        req = fake_request.HTTPRequest.blank('/vnfpkgm/v1/vnf_packages?' +
            query)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        self.assertRaises(tacker_exc.ValidationError, self.controller.index,
                          req)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    @ddt.data(
        {'filter': '(eq,id,fake_value)'},
        {'filter': '(eq,vnfd_id,fake_value)'},
        {'filter': '(eq,operationalState,fake_value)'},
        {'filter': '(eq,softwareImages/size,fake_value)'},
        {'filter': '(gt,softwareImages/createdAt,fake_value)'},
        {'filter': '(eq,softwareImages/minDisk,fake_value)'},
        {'filter': '(eq,softwareImages/minRam,fake_value)'},
    )
    def test_index_filter_invalid_value_type(self, filter_params,
            mock_vnf_list):
        """Test values which doesn't match with attribute data type """
        query = urllib.parse.urlencode(filter_params)
        req = fake_request.HTTPRequest.blank('/vnfpkgm/v1/vnf_packages?' +
            query)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        self.assertRaises(tacker_exc.ValidationError, self.controller.index,
                          req)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    @ddt.data(
        {'fields': 'nonExistentField'},
        {'exclude_fields': 'nonExistentField'}
    )
    def test_index_attribute_selector_invalid_fields(self, params,
            mock_vnf_list):
        query = urllib.parse.urlencode(params)
        req = fake_request.HTTPRequest.blank('/vnfpkgm/v1/vnf_packages?' +
            query)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        self.assertRaises(tacker_exc.ValidationError, self.controller.index,
                          req)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    @ddt.data(
        {'fields': 'softwareImages', 'all_fields': ''},
        {'exclude_fields': 'checksum', 'all_fields': ''},
        {'fields': 'softwareImages', 'exclude_fields': 'checksum'}
    )
    def test_index_attribute_selector_invalid_combination(self, params,
            mock_vnf_list):
        """Test invalid combination of attribute selector parameters """
        query = urllib.parse.urlencode(params)
        req = fake_request.HTTPRequest.blank('/vnfpkgm/v1/vnf_packages?' +
            query)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        self.assertRaises(tacker_exc.ValidationError, self.controller.index,
                          req)

    @mock.patch.object(VnfPackagesList, "get_by_filters")
    @ddt.data(
        {'exclude_default': 'softwareImages'},
        {'all_fields': 'checksum'},
    )
    def test_index_attribute_selector_unexpected_value(self, params,
            mock_vnf_list):
        """Test values with the parameters which doesn't need values. """
        query = urllib.parse.urlencode(params)
        req = fake_request.HTTPRequest.blank('/vnfpkgm/v1/vnf_packages?' +
            query)
        mock_vnf_list.return_value = fakes.return_vnf_package_list()
        self.assertRaises(tacker_exc.ValidationError, self.controller.index,
                          req)

    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    @mock.patch.object(VNFPackageRPCAPI, "delete_vnf_package")
    def test_delete_with_204_status(self, mock_delete_rpc, mock_vnf_by_id):
        vnfpkg_updates = {'operational_state': 'DISABLED'}
        mock_vnf_by_id.return_value = fakes.return_vnfpkg_obj(
            vnf_package_updates=vnfpkg_updates)
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s' % constants.UUID)
        req.headers['Content-Type'] = 'application/json'
        req.method = 'DELETE'
        resp = req.get_response(self.app)
        self.assertEqual(http_client.NO_CONTENT, resp.status_code)

    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_delete_no_vnf_package(self, mock_vnf_by_id):
        req = fake_request.HTTPRequest.blank(
            '/vnfpkgm/v1/vnf_packages/%s' % constants.UUID)
        msg = _("Can not find requested vnf package: %s") % constants.UUID
        mock_vnf_by_id.side_effect = exc.HTTPNotFound(explanation=msg)
        self.assertRaises(exc.HTTPNotFound, self.controller.delete,
                          req, constants.UUID)

    def test_delete_with_invalid_uuid(self):
        req = fake_request.HTTPRequest.blank(
            '/vnfpkgm/v1/vnf_packages/%s' % constants.INVALID_UUID)
        self.assertRaises(exc.HTTPNotFound, self.controller.delete,
                          req, constants.INVALID_UUID)

    @mock.patch.object(objects.VnfPackage, "get_by_id")
    def test_delete_with_operational_state_enabled(self, mock_vnf_by_id):
        req = fake_request.HTTPRequest.blank(
            '/vnfpkgm/v1/vnf_packages/%s' % constants.UUID)
        vnf_package_dict = fakes.fake_vnf_package()
        vnf_package_dict['operational_state'] = \
            fields.PackageOperationalStateType.ENABLED
        vnf_package = objects.VnfPackage(**vnf_package_dict)
        mock_vnf_by_id.return_value = vnf_package
        self.assertRaises(exc.HTTPConflict, self.controller.delete,
                          req, constants.UUID)

    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_delete_with_usage_state_in_use(self, mock_vnf_by_id):
        req = fake_request.HTTPRequest.blank(
            '/vnfpkgm/v1/vnf_packages/%s' % constants.UUID)
        vnf_package_dict = fakes.fake_vnf_package()
        vnf_package_dict['usage_state'] = \
            fields.PackageUsageStateType.IN_USE
        vnf_package = objects.VnfPackage(**vnf_package_dict)
        mock_vnf_by_id.return_value = vnf_package
        self.assertRaises(exc.HTTPConflict, self.controller.delete,
                          req, constants.UUID)

    @mock.patch.object(glance_store, 'store_csar')
    @mock.patch.object(VNFPackageRPCAPI, "upload_vnf_package_content")
    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    @mock.patch.object(vnf_package.VnfPackage, "save")
    def test_upload_vnf_package_content(self, mock_vnf_pack_save,
                                        mock_vnf_by_id,
                                        mock_upload_vnf_package_content,
                                        mock_glance_store):
        updates = {'onboarding_state': 'CREATED',
                   'operational_state': 'DISABLED'}
        vnf_package_dict = fakes.fake_vnf_package(updates)
        vnf_package_obj = objects.VnfPackage(**vnf_package_dict)
        mock_vnf_by_id.return_value = vnf_package_obj
        mock_vnf_pack_save.return_value = vnf_package_obj
        mock_glance_store.return_value = 'location', 0, 'checksum',\
                                         'multihash', 'loc_meta'
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/package_content'
            % constants.UUID)
        req.headers['Content-Type'] = 'application/zip'
        req.method = 'PUT'
        req.body = jsonutils.dump_as_bytes(mock.mock_open())
        resp = req.get_response(self.app)
        mock_glance_store.assert_called()
        self.assertEqual(http_client.ACCEPTED, resp.status_code)

    def test_upload_vnf_package_content_with_invalid_uuid(self):
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/package_content'
            % constants.INVALID_UUID)
        req.headers['Content-Type'] = 'application/zip'
        req.method = 'PUT'
        req.body = jsonutils.dump_as_bytes(mock.mock_open())

        msg = _("Can not find requested vnf package: %s") \
            % constants.INVALID_UUID
        res = self._make_problem_detail('Not Found', msg, 404)
        resp = req.get_response(self.app)
        self.assertEqual(res.text, resp.text)

    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_upload_vnf_package_content_without_vnf_pack(self,
                                                         mock_vnf_by_id):
        msg = _("Can not find requested vnf package: %s") % constants.UUID
        mock_vnf_by_id.side_effect = \
            tacker_exc.VnfPackageNotFound(explanation=msg)
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/package_content' % constants.UUID)
        req.headers['Content-Type'] = 'application/zip'
        req.method = 'PUT'
        req.body = jsonutils.dump_as_bytes(mock.mock_open())

        msg = _("Can not find requested vnf package: %s") % constants.UUID
        res = self._make_problem_detail('Not Found', msg, 404)
        resp = req.get_response(self.app)
        self.assertEqual(res.text, resp.text)

    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_upload_vnf_package_content_with_invalid_status(self,
                                                            mock_vnf_by_id):
        vnf_obj = fakes.return_vnfpkg_obj()
        vnf_obj.__setattr__('onboarding_state', 'ONBOARDED')
        mock_vnf_by_id.return_value = vnf_obj
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/package_content' % constants.UUID)
        req.headers['Content-Type'] = 'application/zip'
        req.method = 'PUT'
        req.body = jsonutils.dump_as_bytes(mock.mock_open())

        msg = _("VNF Package %s onboarding state is not CREATED") \
            % constants.UUID
        res = self._make_problem_detail('Conflict', msg, 409)
        resp = req.get_response(self.app)
        self.assertEqual(res.text, resp.text)

    @mock.patch.object(urllib.request, 'urlopen')
    @mock.patch.object(VNFPackageRPCAPI, "upload_vnf_package_from_uri")
    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    @mock.patch.object(vnf_package.VnfPackage, "save")
    def test_upload_vnf_package_from_uri(self, mock_vnf_pack_save,
                                         mock_vnf_by_id,
                                         mock_upload_vnf_package_from_uri,
                                         mock_url_open):
        body = {"addressInformation": "http://localhost/test_data.zip"}
        updates = {'onboarding_state': 'CREATED',
                   'operational_state': 'DISABLED'}
        vnf_package_dict = fakes.fake_vnf_package(updates)
        vnf_package_obj = objects.VnfPackage(**vnf_package_dict)
        mock_vnf_by_id.return_value = vnf_package_obj
        mock_vnf_pack_save.return_value = vnf_package_obj
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/package_content/upload_from_uri'
            % constants.UUID)
        req.headers['Content-Type'] = 'application/json'
        req.method = 'POST'
        req.body = jsonutils.dump_as_bytes(body)
        resp = req.get_response(self.app)
        self.assertEqual(http_client.ACCEPTED, resp.status_code)

    def test_upload_vnf_package_from_uri_with_invalid_uuid(self):
        body = {"addressInformation": "http://localhost/test_data.zip"}
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/package_content/upload_from_uri'
            % constants.INVALID_UUID)
        self.assertRaises(exc.HTTPNotFound,
                          self.controller.upload_vnf_package_from_uri,
                          req, constants.INVALID_UUID, body=body)

    @mock.patch.object(urllib.request, 'urlopen')
    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_upload_vnf_package_from_uri_without_vnf_pack(self,
                                                          mock_vnf_by_id,
                                                          mock_url_open):
        body = {"addressInformation": "http://localhost/test_data.zip"}
        msg = _("Can not find requested vnf package: %s") % constants.UUID
        mock_vnf_by_id.side_effect = exc.HTTPNotFound(explanation=msg)
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/package_content/upload_from_uri'
            % constants.UUID)
        self.assertRaises(exc.HTTPNotFound,
                          self.controller.upload_vnf_package_from_uri,
                          req, constants.UUID, body=body)

    @mock.patch.object(urllib.request, 'urlopen')
    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_upload_vnf_package_from_uri_with_invalid_status(self,
                                                             mock_vnf_by_id,
                                                             mock_url_open):
        body = {"addressInformation": "http://localhost/test_data.zip"}
        vnf_obj = fakes.return_vnfpkg_obj()
        vnf_obj.__setattr__('onboarding_state', 'ONBOARDED')
        mock_vnf_by_id.return_value = vnf_obj
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/package_content/upload_from_uri'
            % constants.UUID)
        self.assertRaises(exc.HTTPConflict,
                          self.controller.upload_vnf_package_from_uri,
                          req, constants.UUID, body=body)

    @ddt.data("http://test_data.zip", "xyz://github.com/abc/xyz.git",
              "xyz://github.com/abc/xyz")
    def test_upload_vnf_package_from_uri_with_invalid_url(self, invalid_url):
        body = {"addressInformation": invalid_url}
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/package_content/upload_from_uri'
            % constants.UUID)
        self.assertRaises(exc.HTTPBadRequest,
                          self.controller.upload_vnf_package_from_uri,
                          req, constants.UUID, body=body)

    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    @mock.patch.object(vnf_package.VnfPackage, "save")
    def test_patch(self, mock_save, mock_vnf_by_id):
        vnf_package_updates = {'operational_state': 'DISABLED'}
        mock_vnf_by_id.return_value = \
            fakes.return_vnfpkg_obj(vnf_package_updates=vnf_package_updates)

        req_body = {"operationalState": "ENABLED",
                "userDefinedData": {"testKey1": "val01",
                                    "testKey2": "val02", "testkey3": "val03"}}

        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s'
            % constants.UUID)
        req.headers['Content-Type'] = 'application/json'
        req.method = 'PATCH'
        req.body = jsonutils.dump_as_bytes(req_body)
        resp = req.get_response(self.app)

        self.assertEqual(http_client.OK, resp.status_code)
        self.assertEqual(req_body, jsonutils.loads(resp.body))

    def test_patch_with_empty_body(self):
        body = {}
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s'
            % constants.UUID)
        req.headers['Content-Type'] = 'application/json'
        req.method = 'PATCH'
        req.body = jsonutils.dump_as_bytes(body)
        resp = req.get_response(self.app)
        self.assertEqual(http_client.BAD_REQUEST, resp.status_code)

    def test_patch_with_invalid_operational_state(self):
        body = {"operationalState": "DISABLE"}
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s'
            % constants.UUID)
        req.headers['Content-Type'] = 'application/json'
        req.method = 'PATCH'
        req.body = jsonutils.dump_as_bytes(body)
        resp = req.get_response(self.app)
        self.assertEqual(http_client.BAD_REQUEST, resp.status_code)

    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    @mock.patch.object(vnf_package.VnfPackage, "save")
    def test_patch_update_existing_user_data(self, mock_save, mock_vnf_by_id):
        fake_obj = fakes.return_vnfpkg_obj(vnf_package_updates={
            "operational_state": "DISABLED", "onboarding_state": "CREATED",
            "user_data": {"testKey1": "val01", "testKey2": "val02",
            "testKey3": "val03"}})
        mock_vnf_by_id.return_value = fake_obj
        req_body = {"userDefinedData": {"testKey1": "changed_val01",
                                        "testKey2": "changed_val02",
                                        "testKey3": "changed_val03"}}
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s'
            % constants.UUID)
        req.headers['Content-Type'] = 'application/json'
        req.method = 'PATCH'
        req.body = jsonutils.dump_as_bytes(req_body)
        resp = req.get_response(self.app)
        self.assertEqual(http_client.OK, resp.status_code)
        self.assertEqual(req_body, jsonutils.loads(resp.body))

    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    @mock.patch.object(vnf_package.VnfPackage, "save")
    def test_patch_failed_with_same_user_data(self, mock_save,
                                              mock_vnf_by_id):
        vnf_package_updates = {"operational_state": "DISABLED",
            "onboarding_state": "CREATED",
            "user_data": {"testKey1": "val01",
                          "testKey2": "val02",
                          "testkey3": "val03"}}
        req_body = {"userDefinedData": {"testKey1": "val01",
                                        "testKey2": "val02",
                                        "testkey3": "val03"}}
        fake_obj = fakes.return_vnfpkg_obj(
            vnf_package_updates=vnf_package_updates)
        mock_vnf_by_id.return_value = fake_obj

        req = fake_request.HTTPRequest.blank('/vnf_packages/%s'
                                             % constants.UUID)
        self.assertRaises(exc.HTTPConflict,
                          self.controller.patch,
                          req, constants.UUID, body=req_body)

    def test_patch_with_invalid_uuid(self):
        body = {"operationalState": "ENABLED"}
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s'
            % constants.INVALID_UUID)
        exception = self.assertRaises(exc.HTTPNotFound,
                                      self.controller.patch,
                                      req, constants.INVALID_UUID, body=body)
        self.assertEqual(
            "Can not find requested vnf package: %s" % constants.INVALID_UUID,
            exception.explanation)

    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_patch_with_non_existing_vnf_package(self, mock_vnf_by_id):
        body = {"operationalState": "ENABLED"}
        msg = _("Can not find requested vnf package: %s") % constants.UUID
        mock_vnf_by_id.side_effect = exc.HTTPNotFound(explanation=msg)
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s' % constants.UUID)
        exception = self.assertRaises(
            exc.HTTPNotFound, self.controller.patch,
            req, constants.UUID, body=body)
        self.assertEqual(
            "Can not find requested vnf package: %s" % constants.UUID,
            exception.explanation)

    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_patch_failed_with_same_operational_state(self, mock_vnf_by_id):
        vnf_package_updates = {'operational_state': 'DISABLED'}
        mock_vnf_by_id.return_value = \
            fakes.return_vnfpkg_obj(vnf_package_updates=vnf_package_updates)
        body = {"operationalState": "DISABLED",
                "userDefinedData": {"testKey1": "val01",
                                    "testKey2": "val02", "testkey3": "val03"}}
        req = fake_request.HTTPRequest.blank('/vnf_packages/%s'
                                             % constants.UUID)
        self.assertRaises(exc.HTTPConflict,
                          self.controller.patch,
                          req, constants.UUID, body=body)

    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_patch_not_in_onboarded_state(self, mock_vnf_by_id):
        vnf_package_updates = {'onboarding_state': 'CREATED',
            'operational_state': 'DISABLED'}
        mock_vnf_by_id.return_value = fakes.return_vnfpkg_obj(
            vnf_package_updates=vnf_package_updates)
        body = {"operationalState": "DISABLED"}
        req = fake_request.HTTPRequest.blank('/vnf_packages/%s'
                                             % constants.UUID)
        self.assertRaises(exc.HTTPBadRequest,
                          self.controller.patch,
                          req, constants.UUID, body=body)

    @mock.patch.object(VNFPackageRPCAPI, "get_vnf_package_vnfd")
    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    @ddt.data('application/zip', 'text/plain,application/zip',
              'application/zip,text/plain')
    def test_get_vnf_package_vnfd_with_valid_accept_headers(
            self, accept_headers, mock_vnf_by_id, mock_get_vnf_package_vnfd):
        mock_vnf_by_id.return_value = fakes.return_vnfpkg_obj()
        mock_get_vnf_package_vnfd.return_value = fakes.return_vnfd_data()
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/vnfd'
            % constants.UUID)
        req.headers['Accept'] = accept_headers
        req.method = 'GET'
        resp = req.get_response(self.app)
        self.assertEqual(http_client.OK, resp.status_code)

    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_get_vnf_package_vnfd_with_invalid_accept_header(
            self, mock_vnf_by_id):
        mock_vnf_by_id.return_value = fakes.return_vnfpkg_obj()
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/vnfd'
            % constants.UUID)
        req.headers['Accept'] = 'test-invalid-header'
        req.method = 'GET'
        self.assertRaises(exc.HTTPNotAcceptable,
                          self.controller.get_vnf_package_vnfd,
                          req, constants.UUID)

    @mock.patch.object(VNFPackageRPCAPI, "get_vnf_package_vnfd")
    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_get_vnf_package_vnfd_failed_with_bad_request(
            self, mock_vnf_by_id, mock_get_vnf_package_vnfd):
        mock_vnf_by_id.return_value = fakes.return_vnfpkg_obj()
        mock_get_vnf_package_vnfd.return_value = fakes.return_vnfd_data()
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/vnfd'
            % constants.UUID)
        req.headers['Accept'] = 'text/plain'
        req.method = 'GET'
        self.assertRaises(exc.HTTPBadRequest,
                          self.controller.get_vnf_package_vnfd,
                          req, constants.UUID)

    @mock.patch.object(VNFPackageRPCAPI, "get_vnf_package_vnfd")
    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_get_vnf_package_vnfd_for_content_type_text_plain(self,
                                  mock_vnf_by_id,
                                  mock_get_vnf_package_vnfd):
        mock_vnf_by_id.return_value = fakes.return_vnfpkg_obj()
        fake_vnfd_data = fakes.return_vnfd_data(csar_without_tosca_meta=True)
        mock_get_vnf_package_vnfd.return_value = fake_vnfd_data
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/vnfd'
            % constants.UUID)
        req.headers['Accept'] = 'text/plain'
        req.method = 'GET'
        resp = req.get_response(self.app)
        self.assertEqual(http_client.OK, resp.status_code)
        self.assertEqual('text/plain', resp.content_type)
        self.assertEqual(fake_vnfd_data[list(fake_vnfd_data.keys())[0]],
                         resp.text)

    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_get_vnf_package_vnfd_failed_with_invalid_status(
            self, mock_vnf_by_id):
        vnf_package_updates = {
            'onboarding_state': 'CREATED',
            'operational_state': 'DISABLED'
        }
        mock_vnf_by_id.return_value = fakes.return_vnfpkg_obj(
            vnf_package_updates=vnf_package_updates)
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/vnfd'
            % constants.UUID)
        req.headers['Accept'] = 'application/zip'
        req.method = 'GET'
        resp = req.get_response(self.app)
        self.assertEqual(http_client.CONFLICT, resp.status_code)

    def test_get_vnf_package_vnfd_with_invalid_uuid(self):
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/vnfd'
            % constants.INVALID_UUID)
        req.headers['Accept'] = 'application/zip'
        req.method = 'GET'
        exception = self.assertRaises(exc.HTTPNotFound,
                                self.controller.get_vnf_package_vnfd,
                                req, constants.INVALID_UUID)
        self.assertEqual(
            "Can not find requested vnf package: %s" % constants.INVALID_UUID,
            exception.explanation)

    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_get_vnf_package_vnfd_with_non_existing_vnf_packagee(
            self, mock_vnf_by_id):
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/vnfd'
            % constants.UUID)
        req.headers['Accept'] = 'application/zip'
        req.method = 'GET'
        mock_vnf_by_id.side_effect = tacker_exc.VnfPackageNotFound
        self.assertRaises(exc.HTTPNotFound,
                          self.controller.get_vnf_package_vnfd, req,
                          constants.UUID)

    @mock.patch.object(VNFPackageRPCAPI, "get_vnf_package_vnfd")
    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_get_vnf_package_vnfd_failed_with_internal_server_error(
            self, mock_vnf_by_id, mock_get_vnf_package_vnfd):
        mock_vnf_by_id.return_value = fakes.return_vnfpkg_obj()
        mock_get_vnf_package_vnfd.side_effect = tacker_exc.FailedToGetVnfdData
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/vnfd'
            % constants.UUID)
        req.headers['Accept'] = 'application/zip'
        req.method = 'GET'
        resp = req.get_response(self.app)
        self.assertRaises(exc.HTTPInternalServerError,
                          self.controller.get_vnf_package_vnfd,
                          req, constants.UUID)
        self.assertEqual(http_client.INTERNAL_SERVER_ERROR, resp.status_code)

    def test_fetch_vnf_package_content_valid_range(self):
        request = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/package_content/'
            % constants.UUID)
        request.headers["Range"] = 'bytes=10-99'
        range_ = self.controller._get_range_from_request(request, 120)
        self.assertEqual(10, range_.start)
        self.assertEqual(100, range_.end)  # non-inclusive

    def test_fetch_vnf_package_content_invalid_range(self):
        request = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/package_content/'
            % constants.UUID)
        request.headers["Range"] = 'bytes=150-'
        self.assertRaises(exc.HTTPRequestRangeNotSatisfiable,
                          self.controller._get_range_from_request,
                          request, 120)

    def test_fetch_vnf_package_content_invalid_multiple_range(self):
        request = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/package_content/'
            % constants.UUID)
        request.headers["Range"] = 'bytes=10-20,21-30'
        self.assertRaises(exc.HTTPBadRequest,
                          self.controller._get_range_from_request, request,
                          120)

    @mock.patch.object(controller.VnfPkgmController, "_download")
    @mock.patch.object(controller.VnfPkgmController, "_get_range_from_request")
    @mock.patch.object(glance_store, 'get_csar_size')
    @mock.patch.object(controller.VnfPkgmController, '_get_vnf_package')
    @mock.patch.object(vnf_package.VnfPackage, 'save')
    def test_fetch_vnf_package_content(
            self,
            mock_save,
            mock_get_vnf_package,
            mock_get_csar_size,
            mock_get_range,
            mock_download):
        request = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/package_content/'
            % constants.UUID)
        request.headers["Range"] = 'bytes=10-20,21-30'
        request.response = ""
        mock_get_vnf_package.return_value = fakes.return_vnfpkg_obj()
        mock_get_csar_size.return_value = 1000
        mock_get_range.return_value = "10-20, 21-30"
        mock_download.return_value = "Response"
        id = constants.UUID
        result = self.controller.fetch_vnf_package_content(request, id)
        self.assertEqual(result, "Response")
        mock_get_csar_size.assert_called_once_with(
            id, fakes.return_vnfpkg_obj().location_glance_store)
        mock_get_vnf_package.assert_called_once_with(
            id, request)
        mock_get_range.assert_called_once_with(
            request, 1000)

    @mock.patch.object(controller.VnfPkgmController, '_get_vnf_package')
    def test_fetch_vnf_package_content_invalid_onboarding(
            self, mock_get):
        request = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/package_content/'
            % constants.UUID)
        request.headers["Range"] = 'bytes=10-20,21-30'
        request.response = ""
        pkgobj = fakes.return_vnfpkg_obj()
        pkgobj.onboarding_state = fields.PackageOnboardingStateType.PROCESSING
        mock_get.return_value = pkgobj
        id = constants.UUID
        self.assertRaises(exc.HTTPConflict,
                          self.controller.fetch_vnf_package_content, request,
                          id)

    @mock.patch.object(controller.VnfPkgmController, '_get_vnf_package')
    def test_fetch_vnf_package_content_not_present(self, mock_get):
        request = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/package_content/'
            % constants.UUID)
        request.headers["Range"] = 'bytes=10-20,21-30'
        request.response = ""
        pkgobj = fakes.return_vnfpkg_obj()
        mock_get.return_value = pkgobj
        id = constants.UUID
        self.assertRaises(exc.HTTPNotFound,
                          self.controller.fetch_vnf_package_content, request,
                          id)

    @mock.patch.object(controller.VnfPkgmController, "_download")
    @mock.patch.object(controller.VnfPkgmController, "_get_range_from_request")
    @mock.patch.object(controller.VnfPkgmController, '_get_vnf_package')
    def test_fetch_vnf_package_content_with_size(
            self,
            mock_get,
            mock_get_range,
            mock_download):
        request = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/package_content/'
            % constants.UUID)
        request.headers["Range"] = 'bytes=10-20,21-30'
        request.response = ""
        pkgobj = fakes.return_vnfpkg_obj()
        pkgobj.size = 1000
        mock_get_range.return_value = "10-20, 21-30"
        mock_download.return_value = 1000
        mock_get.return_value = pkgobj
        id = constants.UUID
        result = self.controller.fetch_vnf_package_content(request, id)
        self.assertEqual(result, 1000)

    def test_fetch_vnf_package_artifacts_with_invalid_uuid(
            self):
        # invalid_uuid
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/artifacts/%s'
            % (constants.INVALID_UUID, constants.ARTIFACT_PATH))
        req.method = 'GET'
        exception = self.assertRaises(exc.HTTPNotFound,
                          self.controller.fetch_vnf_package_artifacts,
                          req, constants.INVALID_UUID, constants.ARTIFACT_PATH)
        self.assertEqual(
            "Can not find requested vnf package: %s" % constants.INVALID_UUID,
            exception.explanation)

    @mock.patch.object(controller.VnfPkgmController, "_get_csar_path")
    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_fetch_vnf_package_artifacts_with_invalid_path(
            self, mock_vnf_by_id, mock_get_csar_path):
        mock_vnf_by_id.return_value = fakes.return_vnfpkg_obj()
        base_path = os.path.dirname(os.path.abspath(__file__))
        extract_path = os.path.join(base_path, '../../etc/samples/'
                            'sample_vnf_package_csar_in_meta_and_manifest')
        mock_get_csar_path.return_value = extract_path
        # valid_uuid
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/artifacts/%s'
            % (constants.UUID, constants.INVALID_ARTIFACT_PATH))
        req.method = 'GET'
        self.assertRaises(exc.HTTPNotFound,
                          self.controller.fetch_vnf_package_artifacts,
                          req, constants.UUID,
                          constants.INVALID_ARTIFACT_PATH)

    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_fetch_vnf_package_artifacts_with_invalid_range(
            self, mock_vnf_by_id):
        mock_vnf_by_id.return_value = fakes.return_vnfpkg_obj()
        # valid_uuid
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/artifacts/%s'
            % (constants.UUID, constants.ARTIFACT_PATH))
        req.headers['Range'] = 'bytes=150-'
        req.method = 'GET'
        self.assertRaises(exc.HTTPRequestRangeNotSatisfiable,
                          self.controller._get_range_from_request, req,
                          33)

    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_fetch_vnf_package_artifacts_with_invalid_multiple_range(
            self, mock_vnf_by_id):
        mock_vnf_by_id.return_value = fakes.return_vnfpkg_obj()
        # valid_uuid
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/artifacts/%s'
            % (constants.UUID, constants.ARTIFACT_PATH))
        req.headers['Range'] = 'bytes=10-20,21-30'
        req.method = 'GET'
        self.assertRaises(exc.HTTPBadRequest,
                          self.controller._get_range_from_request, req,
                          33)

    @mock.patch.object(controller.VnfPkgmController, "_get_csar_path")
    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_fetch_vnf_package_artifacts_with_range(
            self, mock_vnf_by_id, mock_get_csar_path):
        mock_vnf_by_id.return_value = fakes.return_vnfpkg_obj()
        base_path = os.path.dirname(os.path.abspath(__file__))
        extract_path = os.path.join(base_path, '../../etc/samples/'
                    'sample_vnf_package_csar_in_meta_and_manifest')
        mock_get_csar_path.return_value = extract_path
        # valid_uuid
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/artifacts/%s'
            % (constants.UUID, constants.ARTIFACT_PATH))
        req.headers['Range'] = 'bytes=10-30'
        req.method = 'GET'
        absolute_artifact_path = \
            os.path.join(extract_path, constants.ARTIFACT_PATH)
        with open(absolute_artifact_path, 'rb') as f:
            f.seek(10, 1)
            data = f.read(20)
        artifact_data = \
            self.controller._download_vnf_artifact(
                absolute_artifact_path, 10, 20)
        self.assertEqual(data, artifact_data)

    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_fetch_vnf_package_artifacts_with_non_existing_vnf_package(
            self, mock_vnf_by_id):
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/artifacts/%s'
            % (constants.UUID, constants.ARTIFACT_PATH))
        req.method = 'GET'
        mock_vnf_by_id.side_effect = tacker_exc.VnfPackageNotFound
        self.assertRaises(exc.HTTPNotFound,
                          self.controller.fetch_vnf_package_artifacts, req,
                          constants.UUID, constants.ARTIFACT_PATH)

    @mock.patch.object(controller.VnfPkgmController, "_get_csar_path")
    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_fetch_vnf_package_artifacts_with_non_range(
            self, mock_vnf_by_id, mock_get_csar_path):
        mock_vnf_by_id.return_value = fakes.return_vnfpkg_obj()
        base_path = os.path.dirname(os.path.abspath(__file__))
        extract_path = os.path.join(base_path, '../../etc/samples/'
                    'sample_vnf_package_csar_in_meta_and_manifest')
        mock_get_csar_path.return_value = extract_path
        # valid_uuid
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/artifacts/%s'
            % (constants.UUID, constants.ARTIFACT_PATH))
        req.method = 'GET'
        absolute_artifact_path = \
            os.path.join(extract_path, constants.ARTIFACT_PATH)
        with open(absolute_artifact_path, 'rb') as f:
            data = f.read()
        artifact_data = \
            self.controller._download_vnf_artifact(
                absolute_artifact_path, 0, 34)
        self.assertEqual(data, artifact_data)

    @mock.patch.object(vnf_package.VnfPackage, "get_by_id")
    def test_fetch_vnf_package_artifacts_with_invalid_status(
            self, mock_vnf_by_id):
        vnf_package_updates = {
            'onboarding_state': 'CREATED',
            'operational_state': 'DISABLED'
        }
        mock_vnf_by_id.return_value = fakes.return_vnfpkg_obj(
            vnf_package_updates=vnf_package_updates)
        req = fake_request.HTTPRequest.blank(
            '/vnf_packages/%s/artifacts/%s'
            % (constants.UUID, constants.ARTIFACT_PATH))
        req.method = 'GET'
        self.assertRaises(exc.HTTPConflict,
                          self.controller.fetch_vnf_package_artifacts,
                          req, constants.UUID,
                          constants.ARTIFACT_PATH)



# Copyright (C) 2019 NTT DATA
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tacker.common import exceptions
from tacker import context
from tacker.db.db_sqlalchemy import models
from tacker import objects
from tacker.objects import vnf_package
from tacker.tests.unit.db.base import SqlTestCase
from tacker.tests.unit.objects import fakes as fake_data
from tacker.tests.unit.vnfpkgm import fakes


class TestVnfPackage(SqlTestCase):

    def setUp(self):
        super(TestVnfPackage, self).setUp()
        self.context = context.get_admin_context()
        self.vnf_package = self._create_vnf_package()
        self.vnf_deployment_flavour = self._create_vnf_deployment_flavour()
        self.vnf_artifacts = self._create_vnf_artifacts()

    def _create_vnf_package(self):
        vnfpkgm = objects.VnfPackage(context=self.context,
                                     **fake_data.vnf_package_data)
        vnfpkgm.create()
        return vnfpkgm

    def _create_vnf_deployment_flavour(self):
        flavour_data = fake_data.vnf_deployment_flavour
        flavour_data.update({'package_uuid': self.vnf_package.id})
        vnf_deployment_flavour = objects.VnfDeploymentFlavour(
            context=self.context, **flavour_data)
        vnf_deployment_flavour.create()
        return vnf_deployment_flavour

    def _create_vnf_artifacts(self):
        artifact_data = fake_data.vnf_artifacts
        artifact_data.update({'package_uuid': self.vnf_package.id})
        vnf_artifacts = objects.VnfPackageArtifactInfo(context=self.context,
                                            **artifact_data)
        vnf_artifacts.create()
        return vnf_artifacts

    def test_add_user_defined_data(self):
        vnf_package_db = models.VnfPackage()
        vnf_package_db.update(fakes.fake_vnf_package())
        vnf_package_db.save(self.context.session)
        expected_result = {'key1': 'value1', 'key2': 'value2'}
        result = vnf_package._add_user_defined_data(
            self.context, vnf_package_db.id, vnf_package_db.user_data)
        self.assertEqual(expected_result, result)

    def test_vnf_package_get_by_id(self):
        result = vnf_package._vnf_package_get_by_id(
            self.context, self.vnf_package.id,
            columns_to_join=['vnf_deployment_flavours', 'vnf_artifacts'])
        self.assertEqual(self.vnf_package.id, result.id)
        self.assertTrue(result.vnf_deployment_flavours)
        self.assertTrue(result.vnf_artifacts)

    def test_vnf_package_create(self):
        result = vnf_package._vnf_package_create(self.context,
                                                 fakes.fake_vnf_package())
        self.assertTrue(result.id)

    def test_vnf_package_list(self):
        result = vnf_package._vnf_package_list(
            self.context, columns_to_join=[
                'vnf_deployment_flavours', 'vnf_artifacts'])
        self.assertIsInstance(result, list)
        self.assertTrue(result)

    def test_vnf_package_update(self):
        update = {'user_data': {'test': 'xyz'}}
        result = vnf_package._vnf_package_update(
            self.context, self.vnf_package.id, update)
        self.assertEqual({'test': 'xyz', 'abc': 'xyz'}, result.metadetails)

    def test_destroy_vnf_package(self):
        vnf_package._destroy_vnf_package(self.context,
                                         self.vnf_package.id)
        self.assertRaises(
            exceptions.VnfPackageNotFound,
            objects.VnfPackage.get_by_id, self.context,
            self.vnf_package.id)

    def test_make_vnf_packages_list(self):
        response = vnf_package._vnf_package_list(self.context)
        vnf_pack_list_obj = objects.VnfPackagesList(self.context)
        result = vnf_package._make_vnf_packages_list(
            self.context, vnf_pack_list_obj, response, None)
        self.assertIsInstance(result, objects.VnfPackagesList)
        self.assertTrue(result.objects[0].id)

    def test_patch_user_data_existing_key_with_same_value(self):
        update = {'user_data': {'abc': 'xyz'}}
        result = vnf_package._vnf_package_update(
            self.context, self.vnf_package.id, update)
        self.assertEqual({'abc': 'xyz'}, result.metadetails)

    def test_patch_user_data_existing_key_new_value(self):
        update = {'user_data': {'abc': 'val1'}}
        result = vnf_package._vnf_package_update(
            self.context, self.vnf_package.id, update)
        self.assertEqual({'abc': 'val1'}, result.metadetails)

    def test_patch_user_data_with_new_key_value(self):
        update = {'user_data': {'test': '123'}}
        result = vnf_package._vnf_package_update(
            self.context, self.vnf_package.id, update)
        self.assertEqual({'test': '123', 'abc': 'xyz'}, result.metadetails)

    def test_patch_user_data_add_key_value_and_modify_value(self):
        update = {'user_data': {'test': 'val1', 'abc': 'val2'}}
        result = vnf_package._vnf_package_update(
            self.context, self.vnf_package.id, update)
        self.assertEqual({'test': 'val1', 'abc': 'val2'}, result.metadetails)



# Copyright 2016 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""blob-to-json-text

Revision ID: 6e56d4474b2a
Revises: f958f58e5daa
Create Date: 2016-06-01 09:50:46.296206

"""

import pickle

from oslo_serialization import jsonutils

from alembic import op
import sqlalchemy as sa

from tacker.db import types

# revision identifiers, used by Alembic.
revision = '6e56d4474b2a'
down_revision = 'f958f58e5daa'


def _migrate_data(table, column_name):
    meta = sa.MetaData(bind=op.get_bind())
    t = sa.Table(table, meta, autoload=True)

    for r in t.select().execute():
        stmt = t.update().where(t.c.id == r.id).values(
            {column_name: jsonutils.dump_as_bytes(
                pickle.loads(getattr(r, column_name)))})
        op.execute(stmt)

    op.alter_column(table,
                    column_name,
                    type_=types.Json)


def upgrade(active_plugins=None, options=None):
    _migrate_data('vims', 'placement_attr')
    _migrate_data('vimauths', 'vim_project')
    _migrate_data('vimauths', 'auth_cred')
    _migrate_data('devices', 'placement_attr')



# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tacker import context
from tacker.nfvo.drivers.workflow import workflow_generator
from tacker.tests.unit import base


def get_dummy_ns():
    return {'ns': {'description': '',
                   'tenant_id': 'a81900a92bda40588c52699e1873a92f',
                   'vim_id': '96025dd5-ca16-49f3-9823-958eb04260c4',
                   'vnf_ids': '', 'attributes': {},
                   'nsd_id': 'b8587afb-6099-4f56-abce-572c62e3d61d',
                   'name': 'test_create_ns'},
            'vnfd_details': {'vnf1': {'instances': ['VNF1'],
                             'id': 'dec09ed4-f355-4ec8-a00b-8548f6575a80'},
            'vnf2': {'instances': ['VNF2'],
                     'id': '9f8f2af7-6407-4f79-a6fe-302c56172231'}},
            'placement_attr': {}}


def get_dummy_vnffg_ns():
    return {
        'ns': {
            'description': '',
            'vim_id': '96025dd5-ca16-49f3-9823-958eb04260c4',
            'vnf_ids': '', 'attributes': {},
            'nsd_id': 'b8587afb-6099-4f56-abce-572c62e3d61d',
            'name': 'test_create_ns'},
        'vnfd_details': {
            'vnf1': {'instances': ['VNF1'],
                     'id': 'dec09ed4-f355-4ec8-a00b-8548f6575a80'},
            'vnf2': {'instances': ['VNF2'],
                     'id': '9f8f2af7-6407-4f79-a6fe-302c56172231'}},
        'placement_attr': {},
        'vnffgd_templates': {
            'VNFFG1': {
                'tosca_definitions_version':
                    'tosca_simple_profile_for_nfv_1_0_0',
                'description': 'VNFFG1 descriptor',
                'topology_template': {
                    'node_templates': {
                        'Forwarding_path1': {
                            'type': 'tosca.nodes.nfv.FP.TackerV2',
                            'description': 'creates path inside ns - test',
                            'properties': {
                                'policy': {
                                    'type': 'ACL',
                                    'criteria': [{
                                        'classifier': {
                                            'ip_proto': 6,
                                            'network_src_port_id': {
                                                'get_input': 'net_src_port_id'
                                            },
                                            'ip_dst_prefix': {
                                                'get_input': 'ip_dest_prefix'
                                            },
                                            'destination_port_range': '80-1024'
                                        },
                                        'name': 'block_tcp'}]},
                                'path': [
                                    {'capability': 'CP12',
                                     'forwarder': 'vnf1'},
                                    {'capability': 'CP22',
                                     'forwarder': 'vnf2'}],
                                'id': 51}}},
                    'groups': {
                        'VNFFG1': {
                            'type': 'tosca.groups.nfv.VNFFG',
                            'description': 'HTTP to Corporate Net',
                            'members': ['Forwarding_path1'],
                            'properties': {
                                'version': 1.0,
                                'vendor': 'tacker',
                                'constituent_vnfs': ['vnf1', 'vnf2'],
                                'connection_point': ['CP12', 'CP22'],
                                'number_of_endpoints': 2,
                                'dependent_virtual_link': ['VL1', 'VL2']}
                        }
                    }
                }
            }
        }
    }


def get_dummy_param():
    return {'vnf1': {'substitution_mappings': {'VL1b8587afb-60': {
            'type': 'tosca.nodes.nfv.VL', 'properties': {
                'network_name': 'net_mgmt',
                'vendor': 'tacker'}}, 'requirements': {
                    'virtualLink2': 'VL2b8587afb-60',
                    'virtualLink1': 'VL1b8587afb-60'}, 'VL2b8587afb-60': {
                        'type': 'tosca.nodes.nfv.VL',
                        'properties': {'network_name': 'net0',
                            'vendor': 'tacker'}}}},
            'nsd': {'vl2_name': 'net0', 'vl1_name': 'net_mgmt'}}


def get_dummy_create_workflow():
    return {'std.create_ns_dummy': {'input': ['ns'],
                'tasks': {
                    'wait_vnf_active_VNF2': {
                        'action': 'tacker.show_vnf vnf=<% $.vnf_id_VNF2 %>',
                        'retry': {'count': 10, 'delay': 10,
                            'continue-on': '<% $.status_VNF2 = '
                                           '"PENDING_CREATE" %>',
                            'break-on': '<% $.status_VNF2 = "ERROR" %>'},
                        'publish': {
                            'status_VNF2': '<% task(wait_vnf_active_VNF2).'
                                           'result.vnf.status %>',
                            'mgmt_ip_address_VNF2': ' <% task('
                                                    'wait_vnf_active_VNF2).'
                                                    'result.vnf.'
                                                    'mgmt_ip_address %>'},
                        'on-success': [{
                            'delete_vnf_VNF2': '<% $.status_VNF2='
                                               '"ERROR" %>'}]},
                    'create_ns_VNF2': {
                        'action': 'tacker.create_vnf body=<% $.ns.VNF2 %>',
                        'input': {'body': '<% $.ns.VNF2 %>'},
                        'publish': {
                            'status_VNF2': '<% task(create_ns_VNF2).'
                                           'result.vnf.status %>',
                            'vim_id_VNF2': '<% task(create_ns_VNF2).'
                                           'result.vnf.vim_id %>',
                            'mgmt_ip_address_VNF2': '<% task('
                                                    'create_ns_VNF2).'
                                                    'result.vnf.'
                                                    'mgmt_ip_address %>',
                            'vnf_id_VNF2': '<% task(create_ns_VNF2)'
                                           '.result.vnf.id %>'},
                            'on-success': ['wait_vnf_active_VNF2']},
                    'create_ns_VNF1': {
                        'action': 'tacker.create_vnf body=<% $.ns.VNF1 %>',
                        'input': {'body': '<% $.ns.VNF1 %>'},
                        'publish': {
                            'status_VNF1': '<% task(create_ns_VNF1).'
                                           'result.vnf.status %>',
                            'vnf_id_VNF1': '<% task(create_ns_VNF1).'
                                           'result.vnf.id %>',
                            'mgmt_ip_address_VNF1': '<% task('
                                                    'create_ns_VNF1).'
                                                    'result.vnf.'
                                                    'mgmt_ip_address %>',
                            'vim_id_VNF1': '<% task(create_ns_VNF1).'
                                           'result.vnf.vim_id %>'},
                        'on-success': ['wait_vnf_active_VNF1']},
                    'wait_vnf_active_VNF1': {
                        'action': 'tacker.show_vnf vnf=<% $.vnf_id_VNF1 %>',
                        'retry': {'count': 10, 'delay': 10,
                            'continue-on': '<% $.status_VNF1 = "PENDING_'
                                           'CREATE" %>',
                            'break-on': '<% $.status_VNF1 = "ERROR" %>'},
                        'publish': {
                            'status_VNF1': '<% task(wait_vnf_active_VNF1).'
                                           'result.vnf.status %>',
                            'mgmt_ip_address_VNF1': ' <% task('
                                                    'wait_vnf_active_VNF1).'
                                                    'result.vnf.'
                                                    'mgmt_ip_address %>'},
                        'on-success': [{'delete_vnf_VNF1': '<% $.status_VNF1='
                                                           '"ERROR" %>'}]},
                    'delete_vnf_VNF1': {
                        'action': 'tacker.delete_vnf vnf=<% $.vnf_id_VNF1%>',
                        'input': {'body': {'vnf': {'attributes': {
                                                   'force': False}}}}},
                    'delete_vnf_VNF2': {
                        'action': 'tacker.delete_vnf vnf=<% $.vnf_id_VNF2%>',
                        'input': {'body': {'vnf': {'attributes': {
                                                   'force': False}}}}}},
                'type': 'direct', 'output': {
                    'status_VNF1': '<% $.status_VNF1 %>',
                    'status_VNF2': '<% $.status_VNF2 %>',
                    'mgmt_ip_address_VNF2': '<% $.mgmt_ip_address_VNF2 %>',
                    'mgmt_ip_address_VNF1': '<% $.mgmt_ip_address_VNF1 %>',
                    'vim_id_VNF2': '<% $.vim_id_VNF2 %>',
                    'vnf_id_VNF1': '<% $.vnf_id_VNF1 %>',
                    'vnf_id_VNF2': '<% $.vnf_id_VNF2 %>',
                    'vim_id_VNF1': '<% $.vim_id_VNF1 %>'}},
            'version': '2.0'}


def get_dummy_create_vnffg_ns_workflow():
    return {
        'std.create_ns_dummy': {
            'input': ['ns'],
            'tasks': {
                'wait_vnf_active_VNF2': {
                    'action': 'tacker.show_vnf vnf=<% $.vnf_id_VNF2 %>',
                    'retry': {
                        'count': 10,
                        'delay': 10,
                        'continue-on':
                            '<% $.status_VNF2 = "PENDING_CREATE" %>',
                        'break-on':
                            '<% $.status_VNF2 = "ERROR" %>'},
                    'publish': {
                        'status_VNF2':
                            '<% task(wait_vnf_active_VNF2).result.'
                            'vnf.status %>',
                        'mgmt_ip_address_VNF2':
                            ' <% task(wait_vnf_active_VNF2).result.'
                            'vnf.mgmt_ip_address %>'},
                    'on-success': [
                        {'delete_vnf_VNF2': '<% $.status_VNF2="ERROR" %>'},
                        'create_vnffg_VNFFG1']},
                'create_vnffg_VNFFG1': {
                    'action': 'tacker.create_vnffg body=<% $.ns.VNFFG1 %>',
                    'input': {'body': '<% $.ns.VNFFG1 %>'},
                    'join': 'all',
                    'publish': {
                        'vnffg_id_VNFFG1': '<% task(create_vnffg_VNFFG1).'
                                           'result.vnffg.id %>'}},
                'wait_vnf_active_VNF1': {
                    'action': 'tacker.show_vnf vnf=<% $.vnf_id_VNF1 %>',
                    'retry': {
                        'count': 10,
                        'delay': 10,
                        'continue-on':
                            '<% $.status_VNF1 = "PENDING_CREATE" %>',
                        'break-on':
                            '<% $.status_VNF1 = "ERROR" %>'},
                    'publish': {
                        'status_VNF1':
                            '<% task(wait_vnf_active_VNF1).result.'
                            'vnf.status %>',
                        'mgmt_ip_address_VNF1':
                            ' <% task(wait_vnf_active_VNF1).result.'
                            'vnf.mgmt_ip_address %>'},
                    'on-success': [
                        {'delete_vnf_VNF1': '<% $.status_VNF1="ERROR" %>'},
                        'create_vnffg_VNFFG1']},
                'create_ns_VNF1': {
                    'action': 'tacker.create_vnf body=<% $.ns.VNF1 %>',
                    'input': {'body': '<% $.ns.VNF1 %>'},
                    'publish': {
                        'status_VNF1':
                            '<% task(create_ns_VNF1).result.vnf.status %>',
                        'vnf_id_VNF1':
                            '<% task(create_ns_VNF1).result.vnf.id %>',
                        'mgmt_ip_address_VNF1':
                            '<% task(create_ns_VNF1).result.'
                            'vnf.mgmt_ip_address %>',
                        'vim_id_VNF1':
                            '<% task(create_ns_VNF1).result.vnf.vim_id %>'},
                    'on-success': ['wait_vnf_active_VNF1']},
                'create_ns_VNF2': {
                    'action': 'tacker.create_vnf body=<% $.ns.VNF2 %>',
                    'input': {'body': '<% $.ns.VNF2 %>'},
                    'publish': {
                        'status_VNF2':
                            '<% task(create_ns_VNF2).result.vnf.status %>',
                        'vim_id_VNF2':
                            '<% task(create_ns_VNF2).result.vnf.vim_id %>',
                        'mgmt_ip_address_VNF2':
                            '<% task(create_ns_VNF2).result.'
                            'vnf.mgmt_ip_address %>',
                        'vnf_id_VNF2':
                            '<% task(create_ns_VNF2).result.vnf.id %>'},
                    'on-success': ['wait_vnf_active_VNF2']},
                'delete_vnf_VNF1': {
                    'action': 'tacker.delete_vnf vnf=<% $.vnf_id_VNF1%>',
                    'input': {'body': {'vnf': {'attributes': {
                                               'force': False}}}}},
                'delete_vnf_VNF2': {
                    'action': 'tacker.delete_vnf vnf=<% $.vnf_id_VNF2%>',
                    'input': {'body': {'vnf': {'attributes': {
                                               'force': False}}}}}},
            'type': 'direct',
            'output': {
                'status_VNF1': '<% $.status_VNF1 %>',
                'status_VNF2': '<% $.status_VNF2 %>',
                'mgmt_ip_address_VNF2': '<% $.mgmt_ip_address_VNF2 %>',
                'mgmt_ip_address_VNF1': '<% $.mgmt_ip_address_VNF1 %>',
                'vnffg_id_VNFFG1': '<% $.vnffg_id_VNFFG1 %>',
                'vim_id_VNF2': '<% $.vim_id_VNF2 %>',
                'vnf_id_VNF1': '<% $.vnf_id_VNF1 %>',
                'vnf_id_VNF2': '<% $.vnf_id_VNF2 %>',
                'vim_id_VNF1': '<% $.vim_id_VNF1 %>'}},
        'version': '2.0'}


def dummy_delete_ns_obj():
    return {'vnf_ids': "{'VNF1': '5de5eca6-3e21-4bbd-a9d7-86458de75f0c'}",
            'vnffg_ids': "{}"}


def dummy_delete_vnffg_ns_obj():
    return {'vnf_ids': "{'VNF1': '5de5eca6-3e21-4bbd-a9d7-86458de75f0c'}",
            'vnffg_ids': "{'VNFFG1': '99066f25-3124-44f1-bc5d-bc0bf236b012'}"}


def get_dummy_delete_workflow():
    return {'version': '2.0',
            'std.delete_ns_dummy': {
                'input': ['vnf_id_VNF1'],
                'tasks': {
                    'delete_vnf_VNF1': {
                        'action': 'tacker.delete_vnf vnf=<% $.vnf_id_VNF1%>',
                        'input': {'body': {'vnf': {'attributes': {
                                                   'force': False}}}}}},
                'type': 'direct'}}


def get_dummy_delete_vnffg_ns_workflow():
    return {'version': '2.0',
            'std.delete_ns_dummy': {
                'input': ['vnf_id_VNF1', 'VNFFG1'],
                'tasks': {
                    'delete_vnf_VNF1': {
                        'join': 'all',
                        'action': 'tacker.delete_vnf vnf=<% $.vnf_id_VNF1%>',
                        'input': {'body': {'vnf': {'attributes': {
                                                   'force': False}}}}},
                    'delete_vnffg_VNFFG1': {
                        'action': 'tacker.delete_vnffg vnffg='
                                  '<% $.VNFFG1 %>',
                        'on-success': ['delete_vnf_VNF1']}},
                'type': 'direct'}}


class FakeMistral(object):
    def __init__(self):
        pass


class FakeNFVOPlugin(object):

    def __init__(self, context, client, resource, action):
        self.context = context
        self.client = client
        self.wg = workflow_generator.WorkflowGenerator(resource, action)

    def prepare_workflow(self, **kwargs):
        self.wg.task(**kwargs)


class TestWorkflowGenerator(base.TestCase):
    def setUp(self):
        super(TestWorkflowGenerator, self).setUp()
        self.mistral_client = FakeMistral()

    def test_prepare_workflow_create(self):
        fPlugin = FakeNFVOPlugin(context, self.mistral_client,
                                 resource='ns', action='create')
        fPlugin.prepare_workflow(ns=get_dummy_ns(), params=get_dummy_param())
        wf_def_values = [fPlugin.wg.definition[k] for
            k in fPlugin.wg.definition]
        self.assertIn(get_dummy_create_workflow()['std.create_ns_dummy'],
                      wf_def_values)
        self.assertEqual(get_dummy_create_workflow()['version'],
                         fPlugin.wg.definition['version'])

    def test_prepare_vnffg_ns_workflow_create(self):
        fPlugin = FakeNFVOPlugin(context, self.mistral_client,
                                 resource='ns', action='create')
        fPlugin.prepare_workflow(ns=get_dummy_vnffg_ns(),
                                 params=get_dummy_param())
        wf_def_values = [fPlugin.wg.definition[k] for
            k in fPlugin.wg.definition]
        self.assertIn(
            get_dummy_create_vnffg_ns_workflow()['std.create_ns_dummy'],
            wf_def_values)
        self.assertEqual(
            get_dummy_create_vnffg_ns_workflow()['version'],
            fPlugin.wg.definition['version'])

    def test_prepare_workflow_delete(self):
        fPlugin = FakeNFVOPlugin(context, self.mistral_client,
                                 resource='ns', action='delete')
        fPlugin.prepare_workflow(ns=dummy_delete_ns_obj())
        wf_def_values = [fPlugin.wg.definition[k] for
            k in fPlugin.wg.definition]
        self.assertIn(get_dummy_delete_workflow()['std.delete_ns_dummy'],
                      wf_def_values)
        self.assertEqual(get_dummy_delete_workflow()['version'],
                         fPlugin.wg.definition['version'])

    def test_prepare_vnffg_ns_workflow_delete(self):
        fPlugin = FakeNFVOPlugin(context, self.mistral_client,
                                 resource='ns', action='delete')
        fPlugin.prepare_workflow(ns=dummy_delete_vnffg_ns_obj())
        wf_def_values = [fPlugin.wg.definition[k] for
            k in fPlugin.wg.definition]
        self.assertIn(
            get_dummy_delete_vnffg_ns_workflow()['std.delete_ns_dummy'],
            wf_def_values)
        self.assertEqual(
            get_dummy_delete_vnffg_ns_workflow()['version'],
            fPlugin.wg.definition['version'])



# Copyright 2021 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""Add columns to vnf_lcm_filter

Revision ID: c31f65e0d099
Revises: 3adac34764da
Create Date: 2021-02-03 22:53:36.352774

"""


from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'c31f65e0d099'
down_revision = '3adac34764da'


def upgrade(active_plugins=None, options=None):
    sql_text_length = 65535

    op.add_column(
        'vnf_lcm_filters',
        sa.Column(
            'vnf_products_from_providers', sa.JSON()))

    op.add_column(
        'vnf_lcm_filters',
        sa.Column(
            'operation_states', sa.TEXT(length=sql_text_length),
            sa.Computed(
                "json_unquote(json_extract(`filter`,'$.operationStates'))")))

    op.add_column(
        'vnf_lcm_filters',
        sa.Column(
            'operation_states_len', sa.Integer,
            sa.Computed(
                "ifnull(json_length(`operation_states`),0)")))

    op.add_column(
        'vnf_lcm_filters',
        sa.Column(
            'vnfd_ids', sa.TEXT(length=sql_text_length),
            sa.Computed(
                "json_unquote(json_extract(`filter`,'$.vnfdIds'))")))

    op.add_column(
        'vnf_lcm_filters',
        sa.Column(
            'vnfd_ids_len', sa.Integer,
            sa.Computed(
                "ifnull(json_length(`vnfd_ids`),0)")))

    op.add_column(
        'vnf_lcm_filters',
        sa.Column(
            'vnf_provider', sa.TEXT(length=sql_text_length),
            sa.Computed(
                "(ifnull(json_unquote(json_extract("
                "`vnf_products_from_providers`,'$.vnfProvider')),''))")))

    op.add_column(
        'vnf_lcm_filters',
        sa.Column(
            'vnf_product_name', sa.TEXT(length=sql_text_length),
            sa.Computed(
                "(ifnull(json_unquote(json_extract("
                "`vnf_products_from_providers`,"
                "'$.vnfProducts[0].vnfProductName')),''))")))

    op.add_column(
        'vnf_lcm_filters',
        sa.Column(
            'vnf_software_version', sa.TEXT(length=sql_text_length),
            sa.Computed(
                "(ifnull(json_unquote(json_extract("
                "`vnf_products_from_providers`,'$.vnfProducts[0]"
                ".versions[0].vnfSoftwareVersion')),''))")))

    op.add_column(
        'vnf_lcm_filters',
        sa.Column(
            'vnfd_versions', sa.TEXT(length=sql_text_length),
            sa.Computed(
                "json_unquote(json_extract(`vnf_products_from_providers`,"
                "'$.vnfProducts[0].versions[0].vnfdVersions'))")))

    op.add_column(
        'vnf_lcm_filters',
        sa.Column(
            'vnfd_versions_len', sa.Integer,
            sa.Computed(
                "ifnull(json_length(`vnfd_versions`),0)")))

    op.add_column(
        'vnf_lcm_filters',
        sa.Column(
            'vnf_instance_ids', sa.TEXT(length=sql_text_length),
            sa.Computed(
                "json_unquote(json_extract(`filter`,'$.vnfInstanceIds'))")))

    op.add_column(
        'vnf_lcm_filters',
        sa.Column(
            'vnf_instance_ids_len', sa.Integer,
            sa.Computed(
                "ifnull(json_length(`vnf_instance_ids`),0)")))

    op.add_column(
        'vnf_lcm_filters',
        sa.Column(
            'vnf_instance_names', sa.TEXT(length=sql_text_length),
            sa.Computed(
                "json_unquote(json_extract(`filter`,'$.vnfInstanceNames'))")))

    op.add_column(
        'vnf_lcm_filters',
        sa.Column(
            'vnf_instance_names_len', sa.Integer,
            sa.Computed(
                "ifnull(json_length(`vnf_instance_names`),0)")))



# Copyright (C) 2021 Nippon Telegraph and Telephone Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import os
import shutil
import tempfile

from oslo_utils import uuidutils

from tacker.tests.functional.sol_v2 import paramgen
from tacker.tests.functional.sol_v2 import utils


zip_file_name = os.path.basename(os.path.abspath(".")) + '.zip'
tmp_dir = tempfile.mkdtemp()
vnfd_id = uuidutils.generate_uuid()
utils.make_zip(".", tmp_dir, vnfd_id)

shutil.copy(os.path.join(tmp_dir, zip_file_name), ".")
shutil.rmtree(tmp_dir)

print('#####################################################################\n'
      '# vnfc id should be changed in update req file by show vnf manually.#\n'
      '#####################################################################')

update_min_req = paramgen.update_vnf_min_with_parameter(vnfd_id)
# fake vnfc id, should be get from show vnf
vnfc_ids = ["VDU1-9300a3cb-bd3b-45e4-9967-095040caf827",
    "VDU2-39681281-e6e6-4179-8898-d9ec70f1642a"]
update_max_req = paramgen.update_vnf_max(vnfd_id, vnfc_ids)

with open("update_min_req", "w", encoding='utf-8') as f:
    f.write(json.dumps(update_min_req, indent=2))

with open("update_max_req", "w", encoding='utf-8') as f:
    f.write(json.dumps(update_max_req, indent=2))



# Copyright 2016 Brocade Communications System, Inc.
# All Rights Reserved.
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

from keystoneauth1 import exceptions
from keystoneauth1.identity import v3
from keystoneauth1 import session
from keystoneclient import client
from oslo_config import cfg
from oslo_log import log as logging


LOG = logging.getLogger(__name__)
CONF = cfg.CONF
KEYSTONE_CLIENT_RETRIES = 3
KEYSTONE_RETRY_WAIT = 5


class Keystone(object):
    """Keystone module for OpenStack VIM

    Handles identity operations for a given OpenStack
    instance such as version, session and client
    """

    def get_version(self, base_url=None):
        try:
            keystone_client = client.Client(auth_url=base_url)
        except exceptions.ConnectionError:
            raise
        return keystone_client.version

    def get_session(self, auth_plugin, verify=False):
        ses = session.Session(auth=auth_plugin, verify=verify)
        return ses

    def get_endpoint(self, ses, service_type, region_name=None):
        return ses.get_endpoint(service_type, region_name)

    def initialize_client(self, **kwargs):
        verify = 'True' == kwargs.pop('cert_verify', 'False')
        auth_plugin = v3.Password(**kwargs)
        ses = self.get_session(auth_plugin=auth_plugin, verify=verify)

        client_retries = KEYSTONE_CLIENT_RETRIES
        while client_retries > 0:
            try:
                cli = client.Client('v3', session=ses)
                break
            except exceptions.InternalServerError:
                LOG.warning("keystone service responds with 500 "
                            "(InternalServerError).")
                client_retries = client_retries - 1
                if client_retries == 0:
                    raise
                time.sleep(KEYSTONE_RETRY_WAIT)
        return cli



# Copyright (C) 2020 NTT DATA
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

from oslo_log import log as logging

from tacker.api import views as base
from tacker.common import utils
import tacker.conf
from tacker.objects import fields
from tacker.objects import vnf_instance as _vnf_instance

CONF = tacker.conf.CONF

LOG = logging.getLogger(__name__)


class ViewBuilder(base.BaseViewBuilder):

    FLATTEN_ATTRIBUTES = _vnf_instance.VnfInstance.FLATTEN_ATTRIBUTES

    def _get_links(self, vnf_instance):
        links = {
            "self": {
                "href":
                '{endpoint}/vnflcm/v1/vnf_instances/{id}'.format(
                    endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
                    id=vnf_instance.id)
            }
        }

        if (vnf_instance.instantiation_state ==
                fields.VnfInstanceState.NOT_INSTANTIATED):
            instantiate_link = {
                "instantiate": {
                    "href":
                    '{endpoint}/vnflcm/v1/vnf_instances/{id}/instantiate'
                    .format(endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
                        id=vnf_instance.id)
                }
            }

            links.update(instantiate_link)

        if (vnf_instance.instantiation_state ==
                fields.VnfInstanceState.INSTANTIATED):
            instantiated_state_links = {
                "terminate": {
                    "href":
                    '{endpoint}/vnflcm/v1/vnf_instances/{id}/terminate'
                    .format(endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
                        id=vnf_instance.id)
                },
                "heal": {
                    "href":
                    '{endpoint}/vnflcm/v1/vnf_instances/{id}/heal'
                    .format(endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
                        id=vnf_instance.id)
                },
                "changeExtConn": {
                    "href":
                    '{endpoint}/vnflcm/v1/vnf_instances/{id}/change_ext_conn'
                    .format(endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
                    id=vnf_instance.id)
                }
            }

            links.update(instantiated_state_links)

        return {"_links": links}

    def _get_vim_conn_info(self, vim_connection_info):
        vim_connections = []

        for vim in vim_connection_info:
            access_info = {
                'username': '',
                'region': '',
                'password': '',
                'tenant': ''
            }
            vim_conn = vim

            for key_name in access_info.keys():
                if vim['access_info'].get(key_name):
                    access_info[key_name] = vim['access_info'].get(key_name)

            vim_conn['access_info'] = access_info

            vim_connections.append(vim_conn)

        return vim_connections

    # TODO(esto-aln): This method will be transferred to
    # tacker/api/views/vnf_lcm_op_occs.py in the future
    def _get_lcm_op_occs_links(self, vnf_lcm_op_occs):
        _links = {
            "self": {
                "href":
                '{endpoint}/vnflcm/v1/vnf_lcm_op_occs/{id}'.format(
                    endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
                    id=vnf_lcm_op_occs.id)
            },
            "vnfInstance": {
                "href":
                '{endpoint}/vnflcm/v1/vnf_instances/{id}'.format(
                    endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
                    id=vnf_lcm_op_occs.vnf_instance_id)
            },
            "retry": {
                "href":
                '{endpoint}/vnflcm/v1/vnf_lcm_op_occs/{id}/retry'.format(
                    endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
                    id=vnf_lcm_op_occs.id)
            },
            "rollback": {
                "href":
                '{endpoint}/vnflcm/v1/vnf_lcm_op_occs/{id}/rollback'.format(
                    endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
                    id=vnf_lcm_op_occs.id)
            },
            "grant": {
                "href":
                '{endpoint}/vnflcm/v1/vnf_lcm_op_occs/{id}/grant'.format(
                    endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
                    id=vnf_lcm_op_occs.id)
            },
            "fail": {
                "href":
                '{endpoint}/vnflcm/v1/vnf_lcm_op_occs/{id}/fail'.format(
                    endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
                    id=vnf_lcm_op_occs.id)
            }
        }

        return {"_links": _links}

    def _get_vnf_instance_info(self, vnf_instance):
        vnf_instance_dict = vnf_instance.to_dict()
        vnf_metadata = vnf_instance_dict.pop("vnf_metadata")
        if vnf_metadata:
            vnf_instance_dict.update({"metadata": vnf_metadata})
        vnf_instance_dict = utils.convert_snakecase_to_camelcase(
            vnf_instance_dict)

        links = self._get_links(vnf_instance)

        vnf_instance_dict.update(links)
        return vnf_instance_dict

    # TODO(esto-aln): This method will be transferred to
    # tacker/api/views/vnf_lcm_op_occs.py in the future
    def _get_vnf_lcm_op_occs(self, vnf_lcm_op_occs):
        vnf_lcm_op_occs_dict = vnf_lcm_op_occs.to_dict()
        vnf_lcm_op_occs_dict = utils.convert_snakecase_to_camelcase(
            vnf_lcm_op_occs_dict)
        vnf_lcm_op_occs_dict.pop('errorPoint')

        links = self._get_lcm_op_occs_links(vnf_lcm_op_occs)

        vnf_lcm_op_occs_dict.update(links)
        return vnf_lcm_op_occs_dict

    def create(self, vnf_instance):
        return self._get_vnf_instance_info(vnf_instance)

    def show(self, vnf_instance):
        return self._get_vnf_instance_info(vnf_instance)

    def index(self, vnf_instances):
        return [self._get_vnf_instance_info(vnf_instance)
                for vnf_instance in vnf_instances]

    def _get_subscription_links(self, vnf_lcm_subscription):
        if isinstance(vnf_lcm_subscription.id, str):
            decode_id = vnf_lcm_subscription.id
        else:
            decode_id = vnf_lcm_subscription.id
        return {
            "_links": {
                "self": {
                    "href":
                    '{endpoint}/vnflcm/v1/subscriptions/{id}'.format(
                        endpoint=CONF.vnf_lcm.endpoint_url.rstrip("/"),
                        id=decode_id)}}}

    def _basic_subscription_info(self, vnf_lcm_subscription, filter=None):
        if not filter:
            if 'filter' in vnf_lcm_subscription:
                filter_dict = json.loads(vnf_lcm_subscription.filter)
                return {
                    'id': vnf_lcm_subscription.id,
                    'filter': filter_dict,
                    'callbackUri': vnf_lcm_subscription.callback_uri,
                }
            return {
                'id': vnf_lcm_subscription.id,
                'callbackUri': vnf_lcm_subscription.callback_uri,
            }
        else:
            return {
                'id': vnf_lcm_subscription.id,
                'filter': filter,
                'callbackUri': vnf_lcm_subscription.callback_uri,
            }

    # TODO(esto.aln): To remove all processing that are
    # related to list subscription from vnf_lcm.py and
    # transfer these to vnf_subscriptions.py, but this
    # will be handled in a future patch.
    def _subscription_filter(
            self,
            subscription_data,
            nextpage_opaque_marker,
            paging):
        # filter processing
        lcmsubscription = []
        last_flg = False
        start_num = CONF.vnf_lcm.subscription_num * (paging - 1)
        # Subscription_data counter for comparing
        # subscription_data and start_num
        wk_counter = 0
        for cnt, line in enumerate(subscription_data):
            LOG.debug("cnt %d,line %s" % (cnt, line))

            if start_num > wk_counter:
                wk_counter = wk_counter + 1
            else:
                if (CONF.vnf_lcm.subscription_num > len(
                        lcmsubscription) and nextpage_opaque_marker):
                    # add lcmsubscription
                    vnf_subscription_res = self._basic_subscription_info(
                        line)
                    links = self._get_subscription_links(line)
                    vnf_subscription_res.update(links)
                    lcmsubscription.append(vnf_subscription_res)
                    if CONF.vnf_lcm.subscription_num == len(
                            lcmsubscription):
                        if cnt == len(subscription_data) - 1:
                            last_flg = True
                        break
                elif not nextpage_opaque_marker:
                    # add lcmsubscription
                    vnf_subscription_res = self._basic_subscription_info(
                        line)
                    links = self._get_subscription_links(line)
                    vnf_subscription_res.update(links)
                    lcmsubscription.append(vnf_subscription_res)
                    if CONF.vnf_lcm.subscription_num < len(
                            lcmsubscription):
                        return 400, False
            if cnt == len(subscription_data) - 1:
                last_flg = True

        LOG.debug("len(lcmsubscription) %s" % len(lcmsubscription))
        LOG.debug(
            "CONF.vnf_lcm.subscription_num %s" %
            CONF.vnf_lcm.subscription_num)

        return lcmsubscription, last_flg

    def _get_vnf_lcm_subscription(self, vnf_lcm_subscription, filter=None):
        vnf_lcm_subscription_response = self._basic_subscription_info(
            vnf_lcm_subscription, filter)

        links = self._get_subscription_links(vnf_lcm_subscription)
        vnf_lcm_subscription_response.update(links)

        return vnf_lcm_subscription_response

    def subscription_create(self, vnf_lcm_subscription, filter):
        return self._get_vnf_lcm_subscription(vnf_lcm_subscription, filter)

    # TODO(esto.aln): To remove all processing that are
    # related to list subscription from vnf_lcm.py and
    # transfer these to vnf_subscriptions.py, but this
    # will be handled in a future patch.
    def subscription_list(
            self,
            vnf_lcm_subscriptions,
            nextpage_opaque_marker,
            paging):
        return self._subscription_filter(
            vnf_lcm_subscriptions, nextpage_opaque_marker, paging)

    # TODO(esto.aln): To remove show subscription related processing
    # in vnf_lcm.py. Current processing for show subscription is in
    # vnf_subscriptions.py.
    def subscription_show(self, vnf_lcm_subscriptions):
        return self._get_vnf_lcm_subscription(vnf_lcm_subscriptions)

    # TODO(esto-aln): This method will be transferred to
    # tacker/api/views/vnf_lcm_op_occs.py in the future
    def show_lcm_op_occs(self, vnf_lcm_op_occs):
        return self._get_vnf_lcm_op_occs(vnf_lcm_op_occs)



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_config import cfg
from oslo_utils import importutils

key_manager_opts = [
    cfg.StrOpt('api_class',
               default='tacker.keymgr.barbican_key_manager'
                       '.BarbicanKeyManager',
               help='The full class name of the key manager API class'),
]


def config_opts():
    return [('key_manager', key_manager_opts)]


def API(auth_url, configuration=None):
    conf = configuration or cfg.CONF
    conf.register_opts(key_manager_opts, group='key_manager')

    cls = importutils.import_class(conf.key_manager.api_class)
    return cls(auth_url)



# Copyright 2016 Brocade Communications System, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

from oslo_config import cfg
import yaml

from tacker.plugins.common import constants as evt_constants
from tacker.tests.functional import base
from tacker.tests.utils import read_file

CONF = cfg.CONF


class VnfdTestCreate(base.BaseTackerTest):
    def _test_create_list_delete_tosca_vnfd(self, tosca_vnfd_file, vnfd_name):
        input_yaml = read_file(tosca_vnfd_file)
        tosca_dict = yaml.safe_load(input_yaml)
        tosca_arg = {'vnfd': {'name': vnfd_name,
                              'attributes': {'vnfd': tosca_dict}}}
        vnfd_instance = self.client.create_vnfd(body=tosca_arg)
        self.assertIsNotNone(vnfd_instance)

        vnfds = self.client.list_vnfds().get('vnfds')
        self.assertIsNotNone(vnfds, "List of vnfds are Empty after Creation")

        vnfd_id = vnfd_instance['vnfd']['id']
        self.verify_vnfd_events(
            vnfd_id, evt_constants.RES_EVT_CREATE,
            evt_constants.RES_EVT_ONBOARDED)

        try:
            self.client.delete_vnfd(vnfd_id)
        except Exception:
            assert False, "vnfd Delete failed"
        self.verify_vnfd_events(vnfd_id, evt_constants.RES_EVT_DELETE,
                                evt_constants.RES_EVT_NA_STATE)

    def test_tosca_vnfd(self):
        self._test_create_list_delete_tosca_vnfd('sample-tosca-vnfd.yaml',
                                                 'sample-tosca-vnfd-template')

    def test_tosca_large_vnfd(self):
        self._test_create_list_delete_tosca_vnfd(
            'sample-tosca-vnfd-large-template.yaml',
            'sample-tosca-vnfd-large-template')

    def test_tosca_re_create_delete_vnfd(self):
        self._test_create_list_delete_tosca_vnfd('sample-tosca-vnfd.yaml',
                                                 'test_vnfd')
        time.sleep(1)
        self._test_create_list_delete_tosca_vnfd('sample-tosca-vnfd.yaml',
                                                 'test_vnfd')
